<?php

namespace App\Http\Controllers\API;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use App\Models\CourseCategory;
use App\Models\Course;
use App\Models\Lesson;
use App\Models\PurchasedCourse;
use App\Models\Lectures;
use Illuminate\Support\Facades\DB;
use Carbon\Carbon;

class CourseController extends Controller
{
    public function createCourseCategory(Request $request)
    {

        try {
            $request->validate([
                'code' => 'required|unique:course_categories',
                'name' => 'required|string|min:2',
            ]);

            $course_category = CourseCategory::create([
                'code' => $request->code,
                'name' => $request->name,
            ]);

            return response()->json([
                'success' => true,
                'message' => 'Course Category Inserted Successfully',
                'data' => $course_category,
            ]);
        } catch (\Exception $e) {
            return response()->json(['success' => false, 'message' => $e->getMessage()]);
        }
    }

    public function listCourseCategory(Request $request)
    {

        try {
            if (auth()->user()) {
                $course_category = isset($request->deleted) && ($request->deleted == 1) ? CourseCategory::withTrashed()->get() : CourseCategory::get();
                return response()->json([
                    'success' => true,
                    'data' => $course_category,
                ]);
            } else {
                return response()->json(['success' => false, 'message' => 'User is not Authenticated.']);
            }
        } catch (\Exception $e) {
            return response()->json(['success' => false, 'message' => $e->getMessage()]);
        }
    }

    public function editCourseCategory($id)
    {
        $course_category = CourseCategory::find($id);
        if ($course_category) {
            return response()->json(['success' => true, 'data' => $course_category]);
        }
        return response()->json(['success' => false, 'message' => 'Course Category Not Found!']);
    }

    public function updateCourseCategory(Request $request)
    {
        try {
            $request->validate([
                'id' => 'required|numeric',
                'name' => 'required|string|min:2',
            ]);

            $course_category = CourseCategory::find($request->id);
            if ($course_category) {
                $course_category->name = $request->name;
                $course_category->update();
                return response()->json(['success' => true, 'data' => $course_category, 'message' => 'Course Category Updated.']);
            } else {
                return response()->json(['success' => false, 'message' => 'Course Category Not Found!']);
            }
        } catch (\Exception $e) {
            return response()->json(['success' => false, 'message' => $e->getMessage()]);
        }
    }


    public function deleteCourseCategory($id)
    {
        try {
            $course_category = CourseCategory::withTrashed()->find($id);
            if ($course_category) {
                if(!empty($course_category->deleted_at)){
                    CourseCategory::withTrashed()->find($id)->restore();
                    return response()->json(['success' => true, 'message' => 'Category Restored.']);
                }else{
                    CourseCategory::where('id', $id)->delete();
                    return response()->json(['success' => true, 'message' => 'course Category Deleted.']);
                }
            } else {
                return response()->json(['success' => false, 'message' => 'course Category Not Found!']);
            }
        } catch (\Exception $e) {
            return response()->json(['success' => false, 'message' => $e->getMessage()]);
        }
    }


    // Course Api
    public function createCourse(Request $request)
    {
        try {
            $request->validate([
                'course_category_id' => 'required',
                'code' => 'required|unique:courses',
                'name' => 'required',
                'description' => 'required',
                'image' => 'required|image',
                'status' => 'required',
                'price' => 'required',
                'duration' => 'required',
            ]);

            $path = '';
            if ($request->image && $request->image->isValid()) {
                $path = $this->fileUploadApi('course_images', $request->file('image'));
                // $file_name = time() . '.' . $request->image->extension();
                // $request->file('image')->storeAs('public/course_images',$file_name);
                //     $path = env('APP_URL') ."/storage/course_images/$file_name";
                // $request->image->move(public_path('course_images'), $file_name);
                // $path = env('APP_URL') . "/course_images/$file_name";
                // $path = "public/course_images/$file_name";
            }

            $course = Course::create([
                'course_category_id' => $request->course_category_id,
                'code' => $request->code,
                'name' => $request->name,
                'description' => $request->description,
                'image' => $path,
                'status' => $request->status,
                'price' => $request->price,
                'duration' => $request->duration,
            ]);

            return response()->json([
                'success' => true,
                'message' => 'Course Feed Inserted Successfully',
                'data' => $course,
            ]);
        } catch (\Exception $e) {
            return response()->json(['success' => false, 'message' => $e->getMessage()]);
        }
    }

    public function listCourse()
    {
        try {
            if (auth()->user()) {

                $data['course'] = Course::join('course_categories', 'course_categories.id', '=', 'courses.course_category_id')
                    ->select('courses.*', 'course_categories.name as categories_name')
                    ->whereNull("course_categories.deleted_at")
                    ->get();
                if (sizeof($data['course'])) {

                    foreach ($data['course'] as $c) {
                        $course = PurchasedCourse::where('user_id', auth()->id())->where('course_id', $c->id)->first();
                        if ($course) {
                            $c->course_status = 1;
                        } else {
                            $c->course_status = 0;
                        }
                    }
                    $data['course_pagination'] = $data['course']->toQuery()->paginate(5);

                    return response()->json([
                        'success' => true,
                        'data' => $data,
                    ]);
                }
                return response()->json([
                    'success' => false,
                    'message' => "Course Data not found.",
                ]);
            } else {
                return response()->json(['success' => false, 'message' => 'User is not Authenticated.']);
            }
        } catch (\Exception $e) {

            return response()->json(['success' => false, 'message' => $e->getMessage()]);
        }
    }

    public function listCourseCategorywise($id)
    {
        try {
            if (auth()->user()) {

                $data['course'] = Course::join('course_categories', 'course_categories.id', '=', 'courses.course_category_id')
                    ->select('courses.*', 'course_categories.name as categories_name')
                    ->where('courses.course_category_id', $id)
                    ->whereNull("course_categories.deleted_at")
                    ->get();
                // $data['course_pagination'] = Course::join('course_categories', 'course_categories.id', '=', 'courses.course_category_id')
                // ->select('courses.*', 'course_categories.name as categories_name')->paginate(5);

                return response()->json([
                    'success' => true,
                    'data' => $data,
                ]);
            } else {
                return response()->json(['success' => false, 'message' => 'User is not Authenticated.']);
            }
        } catch (\Exception $e) {

            return response()->json(['success' => false, 'message' => $e->getMessage()]);
        }
    }

    public function viewCourse($id)
    {
        // $data['course'] = Course::join('course_categories', 'courses.id', '=', 'course_categories.id')
        //     ->select('courses.*', 'course_categories.name as categories_name')
        //     ->where('courses.id', $id)
        //     ->first();
        // $data['lesson'] = Lesson::join('course_categories', 'course_categories.id', '=', 'lessons.course_category_id')
        //     ->join('courses', 'courses.id', '=', 'lessons.course_id')
        //     ->select('lessons.*', 'course_categories.name as categories_name', 'courses.name as course_name')
        //     ->where('lessons.course_id', $id)
        //     ->get();
        // $data['lesson'] = Lectures::join('course_categories', 'course_categories.id', '=', 'lectures.course_category_id')
        //     ->join('courses', 'courses.id', '=', 'lectures.course_id')
        //     ->join('lessons', 'lessons.id', '=', 'lectures.lesson_id')
        //     ->select(
        //         'lessons.*',
        //         'course_categories.name as categories_name',
        //         'courses.name as course_name',
        //         DB::raw('SUM(lectures.is_video = 1 ) AS total_video'),
        //         DB::raw('SUM(lectures.is_pdf = 1 ) AS total_pdf')
        //     )
        //     ->where('lessons.course_id', $id)
        //     ->groupBy('lessons.id')
        //     ->get();
        // if ($data['course']) {
        //     return response()->json(['success' => true, 'data' => $data]);
        // }
        // return response()->json(['success' => false, 'message' => 'Course Feed Not Found!']);
        try {
            if (auth()->user()) {
                $data['course'] = Course::join('course_categories', 'courses.course_category_id', '=', 'course_categories.id')
                    ->select('courses.*', 'course_categories.name as categories_name')
                    ->where('courses.id', $id)
                    ->whereNull("course_categories.deleted_at")
                    ->first();
                if ($data['course']) {
                    $course = PurchasedCourse::where('user_id', auth()->id())->where('course_id', $id)->first();
                    if ($course) {
                        $data['course']->course_status = 1;
                    } else {
                        $data['course']->course_status = 0;
                    }
                    $data['lesson'] = Lectures::join('course_categories', 'course_categories.id', '=', 'lectures.course_category_id')
                        ->join('courses', 'courses.id', '=', 'lectures.course_id')
                        ->join('lessons', 'lessons.id', '=', 'lectures.lesson_id')
                        ->select(
                            'lessons.*',
                            'course_categories.name as categories_name',
                            'courses.name as course_name',
                            DB::raw('SUM(lectures.is_video = 1 ) AS total_video'),
                            DB::raw('SUM(lectures.is_pdf = 1 ) AS total_pdf')
                        )
                        ->where('lessons.course_id', $id)
                        ->whereNull("course_categories.deleted_at")
                        ->groupBy('lessons.id')
                        ->get();

                    return response()->json(['success' => true, 'data' => $data]);
                }
                return response()->json(['success' => false, 'message' => 'Course Feed Not Found!']);
            } else {
                return response()->json(['success' => false, 'message' => 'User is not Authenticated.']);
            }
        } catch (\Exception $e) {

            return response()->json(['success' => false, 'message' => $e->getMessage()]);
        }
    }

    public function editCourse($id)
    {
        $course = Course::find($id);
        if ($course) {
            return response()->json(['success' => true, 'data' => $course]);
        }
        return response()->json(['success' => false, 'message' => 'Course Feed Not Found!']);
    }

    public function updateCourse(Request $request)
    {
        try {
            $request->validate([
                'id' => 'required|numeric',
            ]);

            $course = Course::find($request->id);
            $input = $request->all();
            if ($course) {
                if ($request->image && $request->image->isValid()) {
                    $path = $this->fileUploadApi('course_images', $request->file('image'));
                    // $file_name = time() . '.' . $request->image->extension();
                    // $request->file('image')->storeAs('public/course_images',$file_name);
                    // $path = env('APP_URL') ."/storage/course_images/$file_name";
                    // $request->image->move(public_path('course_images'), $file_name);
                    // $path = env('APP_URL') . "/course_images/$file_name";
                    // $path = "public/course_images/$file_name";
                    $input['image'] = $path;
                } else {
                    unset($input['image']);
                }
                $course->update($input);
                return response()->json(['success' => true, 'data' => $course, 'message' => 'Course Updated.']);
            } else {
                return response()->json(['success' => false, 'message' => 'Course Feed Not Found!']);
            }
        } catch (\Exception $e) {
            return response()->json(['success' => false, 'message' => $e->getMessage()]);
        }
    }

    public function deleteCourse($id)
    {
        try {
            $course = Course::find($id);
            if ($course) {
                Lectures::where('course_id', $id)->delete();
                Lesson::where('course_id', $id)->delete();
                PurchasedCourse::where('course_id', $id)->delete();
                Course::where('id', $id)->delete();
                return response()->json(['success' => true, 'message' => 'course Deleted.']);
            } else {
                return response()->json(['success' => false, 'message' => 'course Not Found!']);
            }
        } catch (\Exception $e) {
            return response()->json(['success' => false, 'message' => $e->getMessage()]);
        }
    }

    function course_buyer_list(Request $request){
        $data['data'] = Course::with(['buyers.user'])->where("id",$request->cid)->first();
        return view("course.course_buyer_list",$data);
    }

    // Lesson Api
    public function createLesson(Request $request)
    {
        try {
            $request->validate([
                'course_category_id' => 'required',
                'course_id' => 'required',
                'name' => 'required',
                'description' => 'required',
            ]);

            $lesson = Lesson::create([
                'course_category_id' => $request->course_category_id,
                'course_id' => $request->course_id,
                'name' => $request->name,
                'description' => $request->description,
            ]);

            return response()->json([
                'success' => true,
                'message' => 'Lesson Inserted Successfully',
                'data' => $lesson,
            ]);
        } catch (\Exception $e) {
            return response()->json(['success' => false, 'message' => $e->getMessage()]);
        }
    }

    public function listLesson()
    {
        try {
            if (auth()->user()) {

                $data['lesson'] = Lesson::join('course_categories', 'course_categories.id', '=', 'lessons.course_category_id')
                    ->join('courses', 'courses.id', '=', 'lessons.course_id')
                    ->select('lessons.*', 'course_categories.name as categories_name', 'courses.name as course_name')
                    ->whereNull("course_categories.deleted_at")
                    ->get();
                $data['lesson_pagination'] = Lesson::join('course_categories', 'course_categories.id', '=', 'lessons.course_category_id')
                    ->join('courses', 'courses.id', '=', 'lessons.course_id')
                    ->select('lessons.*', 'course_categories.name as categories_name', 'courses.name as course_name')
                    ->whereNull("course_categories.deleted_at")
                    ->paginate(5);

                return response()->json([
                    'success' => true,
                    'data' => $data,
                ]);
            } else {
                return response()->json(['success' => false, 'message' => 'User is not Authenticated.']);
            }
        } catch (\Exception $e) {

            return response()->json(['success' => false, 'message' => $e->getMessage()]);
        }
    }

    public function listLessonCategorywise($id)
    {
        try {
            if (auth()->user()) {

                $data['lessons'] = Lesson::join('courses', 'courses.id', '=', 'lessons.course_id')
                    ->select('lessons.*', 'courses.name as courses_name')
                    ->where('lessons.course_id', $id)
                    ->get();
                // $data['course_pagination'] = Course::join('course_categories', 'course_categories.id', '=', 'courses.course_category_id')
                // ->select('courses.*', 'course_categories.name as categories_name')->paginate(5);

                return response()->json([
                    'success' => true,
                    'data' => $data,
                ]);
            } else {
                return response()->json(['success' => false, 'message' => 'User is not Authenticated.']);
            }
        } catch (\Exception $e) {

            return response()->json(['success' => false, 'message' => $e->getMessage()]);
        }
    }

    public function editLesson($id)
    {
        $lesson = Lesson::find($id);
        if ($lesson) {
            return response()->json(['success' => true, 'data' => $lesson]);
        }
        return response()->json(['success' => false, 'message' => 'Lesson Feed Not Found!']);
    }

    public function updateLesson(Request $request)
    {
        try {
            $request->validate([
                'id' => 'required|numeric',
            ]);

            $lesson = Lesson::find($request->id);
            $input = $request->all();
            if ($lesson) {
                $lesson->update($input);
                return response()->json(['success' => true, 'data' => $lesson, 'message' => 'Lesson Updated.']);
            } else {
                return response()->json(['success' => false, 'message' => 'Lesson Not Found!']);
            }
        } catch (\Exception $e) {
            return response()->json(['success' => false, 'message' => $e->getMessage()]);
        }
    }

    public function deleteLesson($id)
    {
        try {
            $lesson = Lesson::find($id);
            if ($lesson) {
                Lesson::where('id', $id)->delete();
                return response()->json(['success' => true, 'message' => 'Lesson Deleted.']);
            } else {
                return response()->json(['success' => false, 'message' => 'Lesson Not Found!']);
            }
        } catch (\Exception $e) {
            return response()->json(['success' => false, 'message' => $e->getMessage()]);
        }
    }

    public function freeCourse(Request $request)
    {

        try {
            $request->validate([
                'course_category_id' => 'required',
            ]);
            if (auth()->user()) {
                $free_course = Course::where('status', 'free')
                    ->where('course_category_id', $request->course_category_id)
                    ->orderBy('created_at', 'DESC')
                    ->get();
                if (sizeof($free_course)) {
                    return response()->json([
                        'success' => true,
                        'data' => $free_course,
                    ]);
                } else {
                    return response()->json(['success' => false, 'message' => 'Free Course not available for this category.']);
                }
            } else {
                return response()->json(['success' => false, 'message' => 'User is not Authenticated.']);
            }
        } catch (\Exception $e) {
            return response()->json(['success' => false, 'message' => $e->getMessage()]);
        }
    }


    public function paidCourse(Request $request)
    {

        try {
            $request->validate([
                'course_category_id' => 'required',
            ]);
            if (auth()->user()) {
                $paid_course = Course::where('status', 'paid')
                    ->where('course_category_id', $request->course_category_id)
                    ->orderBy('created_at', 'DESC')
                    ->get();
                if (sizeof($paid_course)) {
                    return response()->json([
                        'success' => true,
                        'data' => $paid_course,
                    ]);
                } else {
                    return response()->json(['success' => false, 'message' => 'Paid Course not available for this category.']);
                }
            } else {
                return response()->json(['success' => false, 'message' => 'User is not Authenticated.']);
            }
        } catch (\Exception $e) {
            return response()->json(['success' => false, 'message' => $e->getMessage()]);
        }
    }
    public function courseHomeList()
    {
        try {
            if (auth()->user()) {
              

                $data['free_course'] = Course::where('status', 'free')
                    ->leftJoin('purchased_courses', function ($join) {
                        $join->on('courses.id', '=', 'purchased_courses.course_id')
                            ->where('purchased_courses.user_id', auth()->id());
                    })
                    ->select('courses.*')
                    ->orderBy('courses.created_at', 'DESC')
                    ->get();
                foreach ($data['free_course'] as $fc) {
                    $purchase_course = PurchasedCourse::where('user_id', auth()->id())->where('course_id', $fc->id)->first();
                    if ($purchase_course) {
                        $fc->course_status = 1;
                    } else {
                        $fc->course_status = 0;
                    }
                }

                $data['paid_course'] = Course::where('status', 'paid')
                    ->orderBy('created_at', 'DESC')
                    ->get();
              $data['banner'] = DB::table('banner')->inRandomOrder()->get();
                foreach ($data['paid_course'] as $pc) {
                    $paid_course = PurchasedCourse::where('user_id', auth()->id())->where('course_id', $pc->id)->first();
                    if ($paid_course) {
                        $pc->course_status = 1;
                    } else {
                        $pc->course_status = 0;
                    }
                }
              

                

                return response()->json([
                    'success' => true,
                    'data' => $data,
                ]);
            } else {
                return response()->json(['success' => false, 'message' => 'User is not Authenticated.']);
            }
        } catch (\Exception $e) {
            return response()->json(['success' => false, 'message' => $e->getMessage()]);
        }
    }

    public function purchaseCourse($id)
    {
        try {
            if (auth()->user()) {
                $purchase_course = PurchasedCourse::where('user_id', auth()->id())
                    ->where('course_id', $id)->first();
                if ($purchase_course) {
                    return response()->json(['success' => false, 'message' => 'You already purchased this course.']);
                } else {
                    $course = Course::find($id);
                    if ($course) {
                        $start_date = Carbon::now()->isoFormat('DD MMMM YYYY');
                        $end_date = Carbon::now()->addMonths($course->duration)->isoFormat('DD MMMM YYYY');

                        $purchase_course_store = PurchasedCourse::create([
                            'user_id' => auth()->id(),
                            'course_id' => $id,
                            'start_date' => $start_date,
                            'end_date' => $end_date,
                        ]);
                        return response()->json(['success' => true, 'data' => $purchase_course_store,]);
                    } else {
                        return response()->json(['success' => false, 'message' => 'Course not available.']);
                    }
                }
            } else {
                return response()->json(['success' => false, 'message' => 'User is not Authenticated.']);
            }
        } catch (\Exception $e) {
            return response()->json(['success' => false, 'message' => $e->getMessage()]);
        }
    }

    public function myPurchaseCourses()
    {
        try {
            if (auth()->user()) {
                $purchase_course = PurchasedCourse::where('user_id', auth()->id())->get();
                if (count($purchase_course) > 0) {
                    $my_courses = DB::table('courses')
                        ->join('purchased_courses', 'courses.id', '=', 'purchased_courses.course_id')
                        ->where('purchased_courses.user_id', auth()->id())
                        ->get();
                    return response()->json(['success' => true, 'data' => $my_courses]);
                } else {
                    return response()->json(['success' => false, 'message' => 'You not Purchased any courses.']);
                }
            } else {
                return response()->json(['success' => false, 'message' => 'User is not Authenticated.']);
            }
        } catch (\Exception $e) {
            return response()->json(['success' => false, 'message' => $e->getMessage()]);
        }
    }

    // lectures Api
    public function createLectures(Request $request)
    {
        try {
            $request->validate([
                'course_category_id' => 'required',
                'course_id' => 'required',
                'lesson_id' => 'required',
                'code' => 'required',
                'name' => 'required|unique:lectures',
                'description' => 'required',
                'language' => 'required',
                'duration' => 'required',
                'image' => 'image',
                'video' => 'mimes:flv,mp4,m3u8,ts,3gp,mov,avi,wmv',
                'pdf' => 'mimes:pdf',
            ]);

            $image_path = '';
            $video_path = '';
            $pdf_path = '';
            $is_video = 0;
            $is_pdf = 0;
            if ($request->image && $request->image->isValid()) {
                $image_path = $this->fileUploadApi('lectures_images', $request->file('image'));
                //   $image_file_name = time() . '.' . $request->image->extension();
                //   $request->file('image')->storeAs('public/lectures_images',$image_file_name);
                //     $image_path = env('APP_URL') ."/storage/lectures_images/$image_file_name";
                //   $request->image->move(public_path('lectures_images'), $image_file_name);
                //   $image_path = env('APP_URL') . "/lectures_images/$image_file_name";
                //   $image_path = "public/lectures_images/$image_file_name";
            }
            if ($request->video && $request->video->isValid()) {
                $video_path = $this->fileUploadApi('lectures_video', $request->file('video'));
                //   $video_file_name = time() . '.' . $request->video->extension();
                //   $request->file('video')->storeAs('public/lectures_video',$video_file_name);
                //     $video_path = env('APP_URL') ."/storage/lectures_video/$video_file_name";
                //   $request->video->move(public_path('lectures_video'), $video_file_name);
                //   $video_path = env('APP_URL') . "/lectures_video/$video_file_name";
                // $video_path = "public/lectures_video/$video_file_name";
                $is_video = 1;
            }
            if ($request->pdf && $request->pdf->isValid()) {
                $pdf_path = $this->fileUploadApi('lectures_pdf', $request->file('pdf'));
                //   $pdf_file_name = time() . '.' . $request->pdf->extension();
                //   $request->file('pdf')->storeAs('public/lectures_pdf',$pdf_file_name);
                //     $pdf_path = env('APP_URL') ."/storage/lectures_pdf/$pdf_file_name";
                //   $request->pdf->move(public_path('lectures_pdf'), $pdf_file_name);
                //   $pdf_path = env('APP_URL') . "/lectures_pdf/$pdf_file_name";
                // $pdf_path = "public/lectures_pdf/$pdf_file_name";
                $is_pdf = 1;
            }

            $lectures = Lectures::create([
                'course_category_id' => $request->course_category_id,
                'course_id' => $request->course_id,
                'lesson_id' => $request->lesson_id,
                'code' => $request->code,
                'name' => $request->name,
                'description' => $request->description,
                'language' => $request->language,
                'image' => $image_path,
                'video' => $video_path,
                'pdf' => $pdf_path,
                'duration' => $request->duration,
                'is_video' => $is_video,
                'is_pdf' => $is_pdf,
            ]);

            return response()->json([
                'success' => true,
                'message' => 'Lectures Inserted Successfully',
                'data' => $lectures,
            ]);
        } catch (\Exception $e) {
            return response()->json(['success' => false, 'message' => $e->getMessage()]);
        }
    }

    public function listLectures()
    {
        try {
            if (auth()->user()) {

                $data['lectures'] = Lectures::join('course_categories', 'course_categories.id', '=', 'lectures.course_category_id')
                    ->join('courses', 'courses.id', '=', 'lectures.course_id')
                    ->join('lessons', 'lessons.id', '=', 'lectures.lesson_id')
                    ->select('lectures.*', 'course_categories.name as categories_name', 'courses.name as course_name', 'lessons.name as lessons_name')
                    ->whereNull("course_categories.deleted_at")
                    ->get();
                $data['lectures_pagination'] = Lectures::join('course_categories', 'course_categories.id', '=', 'lectures.course_category_id')
                    ->join('courses', 'courses.id', '=', 'lectures.course_id')
                    ->join('lessons', 'lessons.id', '=', 'lectures.lesson_id')
                    ->select('lectures.*', 'course_categories.name as categories_name', 'courses.name as course_name', 'lessons.name as lessons_name')
                    ->whereNull("course_categories.deleted_at")
                    ->paginate(5);

                return response()->json([
                    'success' => true,
                    'data' => $data,
                ]);
            } else {
                return response()->json(['success' => false, 'message' => 'User is not Authenticated.']);
            }
        } catch (\Exception $e) {

            return response()->json(['success' => false, 'message' => $e->getMessage()]);
        }
    }

    public function listLecturesCategorywise($id)
    {
        try {
            if (auth()->user()) {

                $data['lectures'] = Lectures::join('course_categories', 'course_categories.id', '=', 'lectures.course_category_id')
                    ->join('courses', 'courses.id', '=', 'lectures.course_id')
                    ->join('lessons', 'lessons.id', '=', 'lectures.lesson_id')
                    ->select('lectures.*', 'course_categories.name as categories_name', 'courses.name as course_name', 'lessons.name as lessons_name')
                    ->where('lessons.id', $id)
                    ->whereNull("course_categories.deleted_at")
                    ->get();
                $data['lectures_pagination'] = Lectures::join('course_categories', 'course_categories.id', '=', 'lectures.course_category_id')
                    ->join('courses', 'courses.id', '=', 'lectures.course_id')
                    ->join('lessons', 'lessons.id', '=', 'lectures.lesson_id')
                    ->select('lectures.*', 'course_categories.name as categories_name', 'courses.name as course_name', 'lessons.name as lessons_name')
                    ->where('lessons.id', $id)
                    ->whereNull("course_categories.deleted_at")
                    ->paginate(5);

                return response()->json([
                    'success' => true,
                    'data' => $data,
                ]);
            } else {
                return response()->json(['success' => false, 'message' => 'User is not Authenticated.']);
            }
        } catch (\Exception $e) {

            return response()->json(['success' => false, 'message' => $e->getMessage()]);
        }
    }


    public function editLectures($id)
    {
        $lectures = Lectures::find($id);
        if ($lectures) {
            return response()->json(['success' => true, 'data' => $lectures]);
        }
        return response()->json(['success' => false, 'message' => 'Lectures Feed Not Found!']);
    }

    public function updateLectures(Request $request)
    {
        try {
            $request->validate([
                'id' => 'required|numeric',
            ]);

            $lectures = Lectures::find($request->id);
            $input = $request->all();
            if ($lectures) {
                if ($request->image && $request->image->isValid()) {
                    $image_path = $this->fileUploadApi('lectures_images', $request->file('image'));
                    $input['image'] = $image_path;
                } else {
                    unset($input['image']);
                }
                if ($request->video && $request->video->isValid()) {
                    $video_path = $this->fileUploadApi('lectures_video', $request->file('video'));
                    $input['video'] = $video_path;
                    $input['is_video'] = 1;
                } else {
                    unset($input['video']);
                }
                if ($request->pdf && $request->pdf->isValid()) {
                    $pdf_path = $this->fileUploadApi('lectures_pdf', $request->file('pdf'));
                    // $pdf_file_name = time() . '.' . $request->pdf->extension();
                    // $request->file('pdf')->storeAs('public/lectures_pdf',$pdf_file_name);
                    // $pdf_path = env('APP_URL') ."/storage/lectures_pdf/$pdf_file_name";
                    // $request->pdf->move(public_path('lectures_pdf'), $file_name);
                    // $pdf_path = env('APP_URL') . "/lectures_pdf/$file_name";
                    // $pdf_path = "public/lectures_pdf/$file_name";
                    $input['pdf'] = $pdf_path;
                    $input['is_pdf'] = 1;
                } else {
                    unset($input['pdf']);
                }
                $lectures->update($input);
                return response()->json(['success' => true, 'data' => $lectures, 'message' => 'Lectures Updated.']);
            } else {
                return response()->json(['success' => false, 'message' => 'Lectures Not Found!']);
            }
        } catch (\Exception $e) {
            return response()->json(['success' => false, 'message' => $e->getMessage()]);
        }
    }

    public function deleteLectures($id)
    {
        try {
            $lectures = Lectures::find($id);
            if ($lectures) {
                Lectures::where('id', $id)->delete();
                return response()->json(['success' => true, 'message' => 'Lectures Deleted.']);
            } else {
                return response()->json(['success' => false, 'message' => 'Lectures Not Found!']);
            }
        } catch (\Exception $e) {
            return response()->json(['success' => false, 'message' => $e->getMessage()]);
        }
    }

    function fileUploadApi($path, $file)
    {
        $filename = pathinfo($file->getClientOriginalName(), PATHINFO_FILENAME);
        $imageName = $filename . rand(1000, 99999) . time() . '.' . $file->getClientOriginalExtension();
        $file->move(base_path() . '/public/' . $path, $imageName);
        return url('/') . '/' . $path . '/' . $imageName;
    }
}
