<?php

namespace App\Http\Controllers\API;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;

use App\Models\HoldingUser;
use App\Models\Holdings;

class HoldingsController extends Controller
{
    public function createHoldingUser(Request $request)
    {
        try {
            $request->validate([
                'name' => 'required',
                'image' => 'required|image',
                'user_type_id' => 'required',
                'holding_url' => 'required',
            ]);

            $path = '';
            if ($request->image && $request->image->isValid()) {
                $path = $this->fileUploadApi('holding_user', $request->file('image'));
            }

            $holding_user = HoldingUser::create([
                'name' => $request->name,
                'description' => $request->description,
                'image' => $path,
                'user_type_id' => $request->user_type_id,
                'holding_url' => $request->holding_url,
            ]);

            return response()->json([
                'success' => true,
                'message' => 'Holding User Inserted Successfully',
                'data' => $holding_user,
            ]);
        } catch (\Exception $e) {
            return response()->json(['success' => false, 'message' => $e->getMessage()]);
        }
    }

    public function listHoldingUser()
    {
        try {
            if(auth()->user()) {

                $data['holding_user'] = HoldingUser::with('userType')->get();
                return response()->json([
                    'success' => true,
                    'data' => $data,
                ]);
            } else {
                return response()->json(['success' => false, 'message' => 'User is not Authenticated.']);
            }
        } catch (\Exception $e) {
            return response()->json(['success' => false, 'message' => $e->getMessage()]);
        }
    }

     public function listHoldingUserBytype(Request $request)
    {
        $request->validate([
            'user_type_id' => 'required',
        ]);
        try {
            //if(auth()->user()) {

                $holding_user = HoldingUser::where('user_type_id',$request->user_type_id)->get();
                $data['holding_user'] = array();
                if(isset($holding_user) && count($holding_user) > 0){
                    foreach($holding_user as $key =>  $val){
                        $data['holding_user'][$key] = array(
                            "id" => $val->id,
                            "name" => $val->name,
                            "description" => $val->description,
                            "image"  => $val->image,
                            "user_type_id" => $val->user_type_id,
                            "created_at" => $val->created_at,
                            "updated_at" => $val->updated_at,
                            "holding_url" => ($val->holding_url != null) ? $val->holding_url : '',
                            );
                    }
                }

                $data['paginated_holding_user'] = HoldingUser::where('user_type_id',$request->user_type_id)->paginate(5);
                return response()->json([
                    'success' => true,
                    'data' => $data,
                ]);
            // } else {
            //     return response()->json(['success' => false, 'message' => 'User is not Authenticated.']);
            // }
        } catch (\Exception $e) {
            return response()->json(['success' => false, 'message' => $e->getMessage()]);
        }
    }

    public function editHoldingUser($id)
    {
        $holding_user = HoldingUser::find($id);
        if($holding_user) {
            return response()->json(['success' => true, 'data' => $holding_user]);
        }
        return response()->json(['success' => false, 'message' => 'Holding User Not Found!']);
    }

    public function updateHoldingUser(Request $request)
    {
        try {
            $request->validate([
                'id' => 'required|numeric',
            ]);

            $holding_user = HoldingUser::find($request->id);
            $input = $request->all();
            if ($holding_user) {
                if ($request->image && $request->image->isValid()) {
                    $path = $this->fileUploadApi('holding_user', $request->file('image'));
                    $input['image'] = $path;
                } else {
                    unset($input['image']);
                }
                $holding_user->update($input);
                return response()->json(['success' => true, 'data' => $holding_user, 'message' => 'User Updated.']);
            } else {
                return response()->json(['success' => false, 'message' => 'User Not Found!']);
            }
        } catch (\Exception $e) {
            return response()->json(['success' => false, 'message' => $e->getMessage()]);
        }
    }

    public function deleteHoldingUser($id)
    {
        try {
            $holding_user = HoldingUser::find($id);
            if ($holding_user) {
                Holdings::where('holding_users_id',$id)->delete();
                HoldingUser::where('id', $id)->delete();
                return response()->json(['success' => true, 'message' => 'Holding User Deleted.']);
            } else {
                return response()->json(['success' => false, 'message' => 'Holding User Not Found!']);
            }
        } catch (\Exception $e) {
            return response()->json(['success' => false, 'message' => $e->getMessage()]);
        }
    }

    public function createHoldings(Request $request)
    {
        try {
            $request->validate([
                'holding_users_id' => 'required',
                'stok' => 'required',
            ]);

            $holdings = Holdings::create([
                'holding_users_id' => $request->holding_users_id,
                'stok' => $request->stok,
                'jan' => $request->jan,
                'feb' => $request->feb,
                'mar' => $request->mar,
                'Apr' => $request->Apr,
                'May' => $request->May,
                'Jun' => $request->Jun,
                'Jul' => $request->Jul,
                'Aug' => $request->Aug,
                'Sep' => $request->Sep,
                'Oct' => $request->Oct,
                'Nov' => $request->Nov,
                'Dec' => $request->Dec,
            ]);

            return response()->json([
                'success' => true,
                'message' => 'Holdings Feed Inserted Successfully',
                'data' => $holdings,
            ]);
        } catch (\Exception $e) {
            return response()->json(['success' => false, 'message' => $e->getMessage()]);
        }
    }

    public function listHoldings()
    {
        try {
            if(auth()->user()) {

                $data['holdings'] = Holdings::join('holding_users', 'holding_users.id', '=', 'holdings.holding_users_id')
                    ->select('holdings.*', 'holding_users.name as holdings_name')
                    ->get();
                $data['paginated_holdings'] = Holdings::join('holding_users', 'holding_users.id', '=', 'holdings.holding_users_id')
                    ->select('holdings.*', 'holding_users.name as holdings_name')
                    ->paginate(5);

                return response()->json([
                    'success' => true,
                    'data' => $data,
                ]);
            } else {
                return response()->json(['success' => false, 'message' => 'User is not Authenticated.']);
            }
        } catch (\Exception $e) {
            return response()->json(['success' => false, 'message' => $e->getMessage()]);
        }
    }

    public function viewHoldings($id)
    {
        try {
            if(auth()->user()) {

                $data['holdings'] = Holdings::join('holding_users', 'holding_users.id', '=', 'holdings.holding_users_id')
                    ->select('holdings.*', 'holding_users.name as holdings_name','holding_users.image','holding_users.id')
                    ->where('holdings.holding_users_id',$id)
                    ->get();
                $data['paginated_holdings'] = Holdings::join('holding_users', 'holding_users.id', '=', 'holdings.holding_users_id')
                    ->select('holdings.*', 'holding_users.name as holdings_name','holding_users.image','holding_users.id')
                    ->where('holdings.holding_users_id',$id)
                    ->paginate(5);

                return response()->json([
                    'success' => true,
                    'data' => $data,
                ]);
            } else {
                return response()->json(['success' => false, 'message' => 'User is not Authenticated.']);
            }
        } catch (\Exception $e) {
            return response()->json(['success' => false, 'message' => $e->getMessage()]);
        }
    }

    public function editHoldings($id)
    {
        $holdings = Holdings::find($id);
        if($holdings) {
            return response()->json(['success' => true, 'data' => $holdings]);
        }
        return response()->json(['success' => false, 'message' => 'Holdings Feed Not Found!']);
    }

    public function updateHoldings(Request $request)
    {
        try {
            $request->validate([
                'id' => 'required|numeric',
            ]);

            $holdings = Holdings::find($request->id);
            $input = $request->all();
            if ($holdings) {
                $holdings->update($input);
                return response()->json(['success' => true, 'data' => $holdings, 'message' => 'Holdings Updated.']);
            } else {
                return response()->json(['success' => false, 'message' => 'Holdings Feed Not Found!']);
            }
        } catch (\Exception $e) {
            return response()->json(['success' => false, 'message' => $e->getMessage()]);
        }
    }

    public function deleteHoldings($id)
    {
        try {
            $holdings = Holdings::find($id);
            if ($holdings) {
                Holdings::where('id', $id)->delete();
                return response()->json(['success' => true, 'message' => 'Holdings Deleted.']);
            } else {
                return response()->json(['success' => false, 'message' => 'Holdings Not Found!']);
            }
        } catch (\Exception $e) {
            return response()->json(['success' => false, 'message' => $e->getMessage()]);
        }
    }



    function fileUploadApi($path, $file)
    {
        $filename = pathinfo($file->getClientOriginalName(), PATHINFO_FILENAME);
        $imageName = $filename . rand(1000, 99999) . time() . '.' . $file->getClientOriginalExtension();
        $file->move(base_path() . '/public/' . $path, $imageName);
        return url('/') . '/' . $path . '/' . $imageName;
    }
}
