<?php

namespace App\Http\Controllers\API;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use App\Models\NewsCategory;
use App\Models\News;
use App\Models\Insight;
use App\Models\Poll;
use App\Models\Ads;
use Illuminate\Support\Facades\DB;

class HomeController extends Controller
{
    //Deshboard

    public function index(Request $request)
    {
        try {
            if (auth()->user()) {
                $auth_id = auth()->id();
                $data['category_count'] = DB::table('user_category')->where('user_id',$auth_id)->where('status',1)->count();
                $data['category'] = DB::table('news_categories')
                    ->leftJoin('user_category',function($join) use ($auth_id){
                     $join->on('news_categories.id', '=', 'user_category.news_category_id')
                    ->where('user_category.user_id',$auth_id)
                    ->where('user_category.status',1);
                })->selectRaw('news_categories.*,CASE WHEN user_category.news_category_id IS NOT NULL THEN 1 ELSE 0 END AS category_status',)
                ->orderBy('news_categories.id', 'ASC')
                ->where("news_categories.deleted_at")
                ->get();


                $data['news_category'] = NewsCategory::get();

                $news = News::join('news_categories', 'news_categories.id', '=', 'news.news_category_id')
                ->leftJoin('bookmarks',function($join) use ($auth_id){
                    $join->on('news.id', '=', 'bookmarks.news_id')
                    ->where('bookmarks.user_id',$auth_id);
                })
                    ->select('news.*', 'news_categories.name as categories_name', DB::raw("CASE WHEN bookmarks.user_id IS NOT NULL THEN 1 ELSE 0 END AS is_bookmark"))
                    ->where("news_categories.deleted_at")
                    ->groupBy('news.id');
                if ($request->search) {
                    $data['news'] = $news->where('title', 'like', '%' . $request->search . '%')
                        ->orWhere('description', 'like', '%' . $request->search . '%')
                        ->orderBy('created_at', 'DESC')->paginate(5);
                } else if ($request->category) {
                    $data['news'] = $news->whereIn('news_category_id', $request->category)->orderBy('created_at', 'DESC')->paginate(5);
                } else {
                    $data['news'] = $news->orderBy('created_at', 'DESC')->paginate(5);
                }



                $data['insight'] = Insight::join('news_categories', 'news_categories.id', '=', 'insights.news_category_id')
                                    ->select('insights.*', 'news_categories.name as categories_name')
                                    ->with('insightImage')
                                    ->orderBy('created_at', 'DESC')
                                    ->where("news_categories.deleted_at")
                                    ->paginate(5);

               

                $data['poll'] = Poll::join('news_categories', 'news_categories.id', '=', 'polls.news_category_id')
                    ->leftJoin('poll_results', 'polls.id', '=', 'poll_results.poll_id')
                    ->select('polls.*', 'news_categories.name as categories_name',DB::raw('SUM(if(poll_results.answer = "0", 1, 0)) AS negetive_count'),DB::raw('SUM(if(poll_results.answer = "1", 1, 0)) AS possitive_count'), DB::raw('SUM(if(poll_results.user_id  = "'.auth()->id().'", 1, 0)) AS status'))
                    ->orderBy('polls.created_at', 'DESC')
                    ->groupBy('polls.id')
                    ->where("news_categories.deleted_at")
                    ->paginate(5);

                $ads = Ads::first();

                if($ads){
                    $data['ads'] = Ads::all()->random(1);
                }

                // $data['ads'] = Ads::all()->random(1);
                // $data['category_count'] = 0;
                // $data['category'] = "";

                // if($request->search){
                //     $news = News::where('title', 'like', '%' . $request->search . '%')
                //     ->orWhere('description', 'like', '%' . $request->search . '%')
                //     ->orderBy('created_at', 'DESC')->get();
                // }else if($request->category){
                //     $data['category_count'] = count($request->category);
                //     $data['category'] = NewsCategory::whereIn('id', $request->category)->get();
                //     $news = News::whereIn('news_category_id', $request->category)->orderBy('created_at', 'DESC')->get();
                // }else{
                //     $news = News::latest()->take(5)->get();
                // }
                // $news_category = NewsCategory::get();


                // $insight = Insight::with('insightImage')->latest()->take(5)->get();
                // $poll = Poll::latest()->take(5)->get();

                // $ads = Ads::all()->random(1);
                // $data['data'] = collect($news)->merge($news_category)->merge($insight)->merge($poll)->merge($ads);

                return response()->json(['success' => true, 'data' => $data,]);
            } else {
                return response()->json(['success' => false, 'message' => 'User is not Authenticated.']);
            }
        } catch (\Exception $e) {
            return response()->json(['success' => false, 'message' => $e->getMessage()]);
        }
    }
}
