<?php

namespace App\Http\Controllers\API;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use App\Models\Insight;
use App\Models\InsightImage;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Validator;

class InsightController extends Controller
{
    //

    public function createInsight(Request $request)
    {
        try {

            $request->validate([
                'news_category_id' => 'required|numeric',
                'title' => 'required',
                'description' => 'required',
                'images' => 'required',
                'images.*' => 'image|mimes:jpeg,png,jpg,gif,svg'
            ]);


            $insight = Insight::create([
                'news_category_id' => $request->news_category_id,
                'title' => $request->title,
                'description' => $request->description,
                //  'images' => json_encode($image_name),
            ]);
            $i_id = $insight->id;
            if ($request->hasfile('images')) {
                $images = $request->file('images');
                foreach ($images as $image) {
                    $image_name = $this->fileUploadApi('insight_images',$image);
                    // $file_name = time() . '.' . $image->extension();
                    // $image->storeAs('public/insight_images', $file_name);
                    // $image_name[] = env('APP_URL') . "/storage/insight_images/$file_name";
                    // $image->move(public_path('insight_images'), $file_name);
                    // $image_name[] = env('APP_URL')."/insight_images/$file_name";
                    // $image_name[] = "public/insight_images/$file_name";
                    InsightImage::create([
                        'insight_id' => $i_id,
                        'image' => $image_name,
                    ]);
                }
            }

            return response()->json([
                'success' => true,
                'message' => 'Insight Inserted Successfully',
                'data' => $insight,
            ]);
        } catch (\Exception $e) {
            return response()->json(['success' => false, 'message' => $e->getMessage()]);
        }
    }

    public function viewInsight($id)
    {
        try {
            if (auth()->user()) {
                $insight = Insight::find($id);
                if ($insight) {
                    return response()->json(['success' => true, 'data' => $insight]);
                }
                return response()->json(['success' => false, 'message' => 'Insight Feed Not Found!']);
            } else {
                return response()->json(['success' => false, 'message' => 'User is not Authenticated.']);
            }
        } catch (\Exception $e) {

            return response()->json(['success' => false, 'message' => $e->getMessage()]);
        }
    }

    public function listInsight()
    {
        try {
            if (auth()->user()) {

                $data['insight'] = Insight::with('insightImage')->join('news_categories', 'news_categories.id', '=', 'insights.news_category_id')->select('insights.*', 'news_categories.name as categories_name')->with('insightImage')->get();
                $data['insight_pagination'] = Insight::join('news_categories', 'news_categories.id', '=', 'insights.news_category_id')->select('insights.*', 'news_categories.name as categories_name')->with('insightImage')->paginate(5);

                return response()->json([
                    'success' => true,
                    'data' => $data,
                ]);
            } else {
                return response()->json(['success' => false, 'message' => 'User is not Authenticated.']);
            }
        } catch (\Exception $e) {

            return response()->json(['success' => false, 'message' => $e->getMessage()]);
        }
    }


    public function editInsight($id)
    {
        $insight = Insight::with('insightImage')->find($id);
        if ($insight) {
            return response()->json(['success' => true, 'data' => $insight]);
        }
        return response()->json(['success' => false, 'message' => 'Insight Not Found!']);
    }

    public function updateInsight(Request $request)
    {
        try {
            $request->validate([
                'id' => 'required|numeric',
            ]);

            $insight = Insight::find($request->id);
            $input = $request->all();
            if ($insight) {
                if ($request->hasfile('images')) {
                    $images = $request->file('images');
                    $image_name = "";
                    foreach ($images as $image) {
                        $image_name = $this->fileUploadApi('insight_images',$image);
                        InsightImage::create([
                            'insight_id' => $insight->id,
                            'image' => $image_name,
                        ]);
                    }
                    
                } 
                $insight->update($input);
                return response()->json(['success' => true, 'data' => $insight, 'message' => 'Insight Updated.']);
            } else {
                return response()->json(['success' => false, 'message' => 'Insight Not Found!']);
            }
        } catch (\Exception $e) {

            return response()->json(['success' => false, 'message' => $e->getMessage()]);
        }
    }

    public function deleteInsight($id)
    {
        try {
            $insight = Insight::find($id);
            if ($insight) {
                InsightImage::where('insight_id', $id)->delete();
                Insight::where('id', $id)->delete();
                return response()->json(['success' => true, 'message' => 'Insight Deleted.']);
            } else {
                return response()->json(['success' => false, 'message' => 'Insight Not Found!']);
            }
        } catch (\Exception $e) {
            return response()->json(['success' => false, 'message' => $e->getMessage()]);
        }
    }

    function deleteInsightImage(Request $request){
        InsightImage::where('id', $request->id)->delete();
        return redirect()->back();
    }

    function fileUploadApi($path,$file)
    {
        $filename = pathinfo($file->getClientOriginalName(), PATHINFO_FILENAME);
        $imageName = $filename.rand(1000,99999).time().'.'.$file->getClientOriginalExtension();
        $file->move(base_path().'/public/'.$path,$imageName);
        return url('/').'/'.$path.'/'.$imageName;
    }
}
