<?php

namespace App\Http\Controllers\API;

use App\Http\Controllers\Controller;
use App\Models\User;
use App\Models\NewsCategory;
use App\Models\News;
use App\Models\Bookmark;
use App\Models\Insight;
use App\Models\Poll;
use App\Models\Ads;
use App\Models\InsightImage;
use App\Models\NewsCount;
use App\Traits\Fcm;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Validator;
use Illuminate\Http\Request;

class NewsController extends Controller
{
    use Fcm;
    // create new news Category
    public function createNewsCategory(Request $request)
    {

        try {
            $request->validate([
                'name' => 'required|string|min:2|max:100|unique:news_categories',
                'image' => 'required|image',
            ]);

            $image_path = '';
            if ($request->image && $request->image->isValid()) {
                $image_path = $this->fileUploadApi('news_category_images', $request->file('image'));
            }

            $news_category = NewsCategory::create([
                'name' => $request->name,
                'image' => $image_path,
            ]);

            return response()->json([
                'success' => true,
                'message' => 'News Category Inserted Successfully',
                'data' => $news_category,
            ]);
        } catch (\Exception $e) {
            return response()->json(['success' => false, 'message' => $e->getMessage()]);
        }
    }

    public function listNewsCategory(Request $request)
    {
        try {
            if (auth()->user()) {
                $data['category_count'] = 0;
                $data['category'] = isset($request->deleted) && ($request->deleted == 1) ? NewsCategory::withTrashed()->get() : NewsCategory::get();
                if ($request->category) {
                    $data['category_count'] = count($request->category);
                    // $data['category'] = NewsCategory::get();
                    foreach ($data['category'] as $cat) {
                        if (in_array($cat->id, $request->category)) {
                            $cat->status = 1;
                        } else {
                            $cat->status = 0;
                        }
                    }
                }
                // $data['news_category'] = NewsCategory::get();
                return response()->json([
                    'success' => true,
                    'data' => $data,
                ]);
            } else {
                return response()->json(['success' => false, 'message' => 'User is not Authenticated.']);
            }
        } catch (\Exception $e) {
            return response()->json(['success' => false, 'message' => $e->getMessage()]);
        }
    }

    public function listSelectedNewsCategory(Request $request)
    {
        try {

            $request->validate([
                'ids' => 'required|array',
            ]);

            $ids = $request->ids;
            if (auth()->user()) {
                $news = News::join('news_categories', 'news_categories.id', '=', 'news.news_category_id')
                    ->select('news.*', 'news_categories.name as categories_name', 'news_categories.image as categories_image')
                    ->whereIn('news.news_category_id', $ids)->get();
                return response()->json([
                    'success' => true,
                    'data' => $news,
                ]);
            } else {
                return response()->json(['success' => false, 'message' => 'User is not Authenticated.']);
            }
        } catch (\Exception $e) {
            return response()->json(['success' => false, 'message' => $e->getMessage()]);
        }
    }

    public function editNewsCategory($id)
    {
        $news_category = NewsCategory::find($id);
        if ($news_category) {
            return response()->json(['success' => true, 'data' => $news_category]);
        }
        return response()->json(['success' => false, 'message' => 'News Category Not Found!']);
    }

    public function updateNewsCategory(Request $request)
    {
        try {
            $request->validate([
                'id' => 'required|numeric',
                'name' => 'required|string|min:2|max:100',
                'image' => 'required|image',
            ]);

            $news_category = NewsCategory::find($request->id);
            $input = $request->all();
            if ($news_category) {
                if ($request->image && $request->image->isValid()) {
                    $path = $this->fileUploadApi('news_category_images', $request->file('image'));
                    $input['image'] = $path;
                } else {
                    unset($input['image']);
                }
                $news_category->update($input);
                return response()->json(['success' => true, 'data' => $news_category, 'message' => 'News Category Updated.']);
            } else {
                return response()->json(['success' => false, 'message' => 'News Category Not Found!']);
            }
        } catch (\Exception $e) {
            return response()->json(['success' => false, 'message' => $e->getMessage()]);
        }
    }



    public function deleteNewsCategory($id)
    {
        try {
            $news_category = NewsCategory::withTrashed()->find($id);
            if ($news_category) {
                if(!empty($news_category->deleted_at)){
                    NewsCategory::withTrashed()->find($id)->restore();
                    return response()->json(['success' => true, 'message' => 'News Category Restored.']);
                }else{
                    NewsCategory::find($id)->delete();
                    return response()->json(['success' => true, 'message' => 'News Category Deleted.']);
                }
            } else {
                return response()->json(['success' => false, 'message' => 'News Category Not Found!']);
            }
        } catch (\Exception $e) {

            return response()->json(['success' => false, 'message' => $e->getMessage()]);
        }
    }


    // News Configration

    public function editNewsCount($id)
    {
        $news_count = NewsCount::find($id);
        if ($news_count) {
            return response()->json(['success' => true, 'data' => $news_count]);
        }
        return response()->json(['success' => false, 'message' => 'News Count Not Found!']);
    }

    public function updateNewsCount(Request $request)
    {
        try {
            $request->validate([
                'id' => 'required|numeric',
                'poll_count' => 'required|numeric',
                'insight_count' => 'required|numeric',
                'ads_count' => 'required|numeric',
            ]);

            $news_count = NewsCount::find($request->id);
            $input = $request->all();
            if ($news_count) {
                $news_count->update($input);
                return response()->json(['success' => true, 'data' => $news_count, 'message' => 'News Count Updated.']);
            } else {
                return response()->json(['success' => false, 'message' => 'News Count Not Found!']);
            }
        } catch (\Exception $e) {
            return response()->json(['success' => false, 'message' => $e->getMessage()]);
        }
    }

    // news api
    public function createNews(Request $request)
    {
        try {
            $request->validate([
                'title' => 'required|string|min:2',
                'description' => 'required',
                'news_category_id' => 'required|numeric',
                'image' => 'required|image',
                'link' => 'nullable',
            ]);

            $path = '';
            if ($request->image && $request->image->isValid()) {
                $path = $this->fileUploadApi('news_images', $request->file('image'));
            }

            $news = News::create([
                'title' => $request->title,
                'description' => $request->description,
                'news_category_id' => $request->news_category_id,
                'image' => $path,
                'link' => $request->link,
            ]);

            return response()->json([
                'success' => true,
                'message' => 'News Feed Inserted Successfully',
                'data' => $news,
            ]);
        } catch (\Exception $e) {
            return response()->json(['success' => false, 'message' => $e->getMessage()]);
        }
    }

    public function listNews()
    {
        try {
            //if (auth()->user()) {
                $auth_id = auth()->user();

                $userIntrustedCategory = array_map(function($v){return $v->news_category_id;},DB::table("user_category")->where("user_id",$auth_id)->where('status',1)->get('news_category_id')->toArray());

                $data['news'] = News::join('news_categories', 'news_categories.id', '=', 'news.news_category_id')
                    // ->leftJoin('bookmarks', 'news.id', '=', 'bookmarks.news_id')
                    ->leftJoin('bookmarks', function ($join) use ($auth_id) {
                        $join->on('news.id', '=', 'bookmarks.news_id')
                            ->where('bookmarks.user_id', $auth_id);
                    })
                    ->select('news.*', 'news_categories.name as categories_name', DB::raw("CASE WHEN bookmarks.user_id IS NOT NULL THEN 1 ELSE 0 END AS is_bookmark"))
                    ->when((count($userIntrustedCategory) > 0),function($q) use($userIntrustedCategory) { return $q->whereIn('news.news_category_id',$userIntrustedCategory); })
                    ->whereNull('news_categories.deleted_at')
                    ->groupBy('news.id')
                    ->orderBy('news.id', 'DESC')
                    ->get();
                $fnews = $data['news'];
                $news = $data['news']->toArray();


                //$i = Insight::with('insightImage')->orderBy('news.id', 'DESC')->get();
                $insight = [];
                //if ($i) {
                    $insight = Insight::join('news_categories', 'news_categories.id', '=', 'insights.news_category_id')
                                        ->select('insights.*', 'news_categories.name as categories_name')->with('insightImage')
                                        ->when((count($userIntrustedCategory) > 0),function($q) use($userIntrustedCategory) { return $q->whereIn('insights.news_category_id',$userIntrustedCategory); })
                                        ->whereNull('news_categories.deleted_at')
                                        ->orderBy('id', 'DESC')->get();
               // }

               // $p = Poll::get();
                $poll = [];
                //if ($p) {
                    $poll = Poll::join('news_categories', 'news_categories.id', '=', 'polls.news_category_id')
                        ->leftJoin('poll_results', 'polls.id', '=', 'poll_results.poll_id')
                        ->select('polls.*', 'news_categories.name as categories_name', DB::raw('SUM(if(poll_results.answer = "0", 1, 0)) AS negetive_count'), DB::raw('SUM(if(poll_results.answer = "1", 1, 0)) AS possitive_count'), DB::raw('SUM(if(poll_results.user_id  = "' . auth()->id() . '", 1, 0)) AS status'))
                        ->when((count($userIntrustedCategory) > 0),function($q) use($userIntrustedCategory) { return $q->whereIn('polls.news_category_id',$userIntrustedCategory); })
                        ->whereNull('news_categories.deleted_at')
                        ->groupBy('polls.id')
                        ->orderBy('polls.id', 'DESC')
                        ->get();
                //}
                $a = Ads::get();
                $ads = [];
                if ($a) {
                    $ads = Ads::orderBy('id','desc')->get();
                }
                // $data['data'] = collect($data['news'])->merge($insight)->merge($poll)->merge($ads);
                $news_count_row = NewsCount::first();
                $poll_count = $news_count_row->poll_count;
                $insight_count = $news_count_row->insight_count;
                $ads_count = $news_count_row->ads_count;

                $poll_added = "0";
                $insight_added = "0";
                $ads_added = "0";

                $poll_added_s = "0";
                $insight_added_s = "0";
                $ads_added_s = "0";

                foreach ($news as $n) {
                    $data_final[] = $n;
                    $poll_added_s++;
                    if ($poll_added_s == $poll_count && count($poll) > $poll_added) {
                        $data_final[] = $poll[$poll_added];
                        $poll_added++;
                        $poll_added_s = 0;
                    }
                    $insight_added_s++;
                    if ($insight_added_s == $insight_count && count($insight) > $insight_added) {
                        $data_final[] = $insight[$insight_added];
                        $insight_added++;
                        $insight_added_s = 0;
                    }
                    $ads_added_s++;
                    if ($ads_added_s == $ads_count && count($ads) > $ads_added) {
                        $data_final[] = $ads[$ads_added];
                        $ads_added++;
                        $ads_added_s = 0;
                    }
                }

                array_push($news, $insight, $poll, $ads);
                $data = $news;

                return response()->json([
                    'success' => true,
                    'data' => $data_final ?? [],
                    //  'news' => $fnews
                ]);
           /*  } else {
                return response()->json(['success' => false, 'message' => 'User is not Authenticated.']);
            } */
        } catch (\Exception $e) {

            return response()->json(['success' => false, 'message' => $e->getMessage()]);
        }
    }

    public function viewNews($id)
    {
        $news = News::find($id);
        if ($news) {
            return response()->json(['success' => true, 'data' => $news]);
        }
        return response()->json(['success' => false, 'message' => 'News Feed Not Found!']);
    }

    public function editNews($id)
    {
        $news = News::find($id);
        if ($news) {
            return response()->json(['success' => true, 'data' => $news]);
        }
        return response()->json(['success' => false, 'message' => 'News Feed Not Found!']);
    }

    public function updateNews(Request $request)
    {
        try {
            $request->validate([
                'id' => 'required|numeric',
            ]);

            $news = News::find($request->id);
            $input = $request->all();
            if ($news) {
                if ($request->image && $request->image->isValid()) {
                    $path = $this->fileUploadApi('news_images', $request->file('image'));
                    // $file_name = time() . '.' . $request->image->extension();
                    // $request->file('image')->storeAs('public/news_images',$file_name);
                    // $path = env('APP_URL') ."/storage/news_images/$file_name";
                    // $request->image->move(public_path('news_images'), $file_name);
                    // $path = env('APP_URL') . "/news_images/$file_name";
                    // $path = "public/news_images/$file_name";
                    $input['image'] = $path;
                } else {
                    unset($input['image']);
                }
                $news->update($input);
                return response()->json(['success' => true, 'data' => $news, 'message' => 'News Updated.']);
            } else {
                return response()->json(['success' => false, 'message' => 'News Feed Not Found!']);
            }
        } catch (\Exception $e) {

            return response()->json(['success' => false, 'message' => $e->getMessage()]);
        }
    }

    public function deleteNews($id,$article)
    {
        try {
            if($article == 1){
                bookmark::where("news_id",$id)->delete();
                News::where('id', $id)->delete();
                return response()->json(['success' => true, 'message' => 'News Deleted.']);
            }
            if($article == 2){
                PollResult::where("poll_id",$id)->delete();
                Poll::where('id', $id)->delete();
                return response()->json(['success' => true, 'message' => 'Poll Deleted.']);
            }
            if($article == 3){
                InsightImage::where('insight_id', $id)->delete();
                Insight::where('id', $id)->delete();
                return response()->json(['success' => true, 'message' => 'News Deleted.']);
            }
            if($article == 4){
                Ads::where('id', $id)->delete();
                return response()->json(['success' => true, 'message' => 'News Deleted.']);
            }
        } catch (\Exception $e) {
            return response()->json(['success' => false, 'message' => $e->getMessage()]);
        }
    }

    public function listNewsCategoryWise(Request $request)
    {
        try {
            $request->validate([
                'news_category_id' => 'required|numeric',
            ]);

            if (auth()->user()) {
                $auth_id = auth()->id();
                // $news = News::where('news_category_id', $request->news_category_id)->paginate(5);
                $news = News::join('news_categories', 'news_categories.id', '=', 'news.news_category_id')
                    ->leftJoin('bookmarks', function ($join) use ($auth_id) {
                        $join->on('news.id', '=', 'bookmarks.news_id')
                            ->where('bookmarks.user_id', $auth_id);
                    })
                    ->join('user_category', 'news_categories.id', '=', 'user_category.news_category_id')
                    ->select('news.*', 'news_categories.name as categories_name', DB::raw("CASE WHEN bookmarks.user_id IS NOT NULL THEN 1 ELSE 0 END AS is_bookmark"))
                    ->where('news.news_category_id', '=', $request->news_category_id)
                    ->groupBy('news.id')
                    ->orderBy('news.id', 'DESC')
                    ->get();

                if (count($news) > 0) {
                    return response()->json([
                        'success' => true,
                        'data' => $news,
                    ]);
                } else {
                    return response()->json(['success' => false, 'message' => 'News Not Found!']);
                }
            } else {
                return response()->json(['success' => false, 'message' => 'User is not Authenticated.']);
            }
        } catch (\Exception $e) {
            return response()->json(['success' => false, 'message' => $e->getMessage()]);
        }
    }

    public function bookmarkNews($id)
    {
        try {
            if (auth()->user()) {
                $news = News::find($id);
                if ($news) {
                    $is_bookmark = Bookmark::where('user_id', auth()->id())
                        ->where('news_id', $news->id)->get();
                    if (count($is_bookmark) > 0) {
                        Bookmark::where('user_id', auth()->id())
                            ->where('news_id', $news->id)->delete();
                        return response()->json(['success' => true, 'message' => 'Removed News to Bookmark.']);
                    } else {
                        $bookmark = Bookmark::create([
                            'user_id' => auth()->id(),
                            'news_id' => $news->id,
                        ]);
                        return response()->json(['success' => true, 'data' => $bookmark, 'message' => 'Added News to Bookmark.']);
                    }
                } else {
                    return response()->json(['success' => false, 'message' => 'News Feed Not Found!']);
                }
            } else {
                return response()->json(['success' => false, 'message' => 'User is not Authenticated.']);
            }
        } catch (\Exception $e) {
            return response()->json(['success' => false, 'message' => $e->getMessage()]);
        }
    }

    public function listBookmarkNews()
    {
        try {
            if (auth()->user()) {
                $is_bookmark = Bookmark::where('user_id', auth()->id())->get();
                if (count($is_bookmark) > 0) {
                    $bookmark_news = DB::table('news')
                        ->join('bookmarks', 'news.id', '=', 'bookmarks.news_id')
                        ->join('news_categories', 'news_categories.id', '=', 'news.news_category_id')
                        ->select('news.*', 'news_categories.name as categories_name')
                        ->whereNull("news_categories.deleted_at")
                        ->where('bookmarks.user_id', auth()->id())->orderBy("news.id","desc")->get();
                    return response()->json(['success' => true, 'data' => $bookmark_news, 'message' => 'Listed News this Bookmarked You.']);
                } else {
                    return response()->json(['success' => true, 'message' => 'Bookmark not found.']);
                }
            } else {
                return response()->json(['success' => false, 'message' => 'User is not Authenticated.']);
            }
        } catch (\Exception $e) {
            return response()->json(['success' => false, 'message' => $e->getMessage()]);
        }
    }

    function updateUsercategory(Request $request)
    {
        // $jsonobj = '{"id":1,"status":0},{"id":2,"status":1},{"id":3,"status":0},{"id":4,"status":1}';
        // foreach ($request->news_category_id as $nci) {
        //     // return $nci;
        //     foreach (json_decode($nci, TRUE) as $c) {
        //         return $c['status'];
        //     }
        // }

        try {
            $request->validate([
                'news_category_id' => 'required',
            ]);

            if (auth()->user()) {
                foreach ($request->news_category_id as $nci) {
                    foreach (json_decode($nci, TRUE) as $c) {
                        $uc = DB::table('user_category')->where('news_category_id', $c['id'])->where('user_id', auth()->id())->first();
                        if ($uc) {
                            DB::table('user_category')->where('news_category_id', $c['id'])->where('user_id', auth()->id())->update(['status' => $c['status']]);
                            // $uc->status = $c['status'];
                            // $uc->save();
                        } else {
                            DB::table('user_category')->insert([
                                'user_id' => auth()->id(),
                                'news_category_id' => $c['id'],
                                'status' => $c['status'],
                            ]);
                        }
                    }
                }
                return response()->json(['success' => true, 'message' => 'User Category Updated Successfully.']);
            } else {
                return response()->json(['success' => false, 'message' => 'User is not Authenticated.']);
            }
        } catch (\Exception $e) {
            return response()->json(['success' => false, 'message' => $e->getMessage()]);
        }
    }

    function fileUploadApi($path, $file)
    {
        $filename = pathinfo($file->getClientOriginalName(), PATHINFO_FILENAME);
        $imageName = $filename . rand(1000, 99999) . time() . '.' . $file->getClientOriginalExtension();
        $file->move(base_path() . '/public/' . $path, $imageName);
        return url('/') . '/' . $path . '/' . $imageName;
    }


    function sendFcmNotification(Request $request){

        $dataId = $request->dataId;
        $articleType = $request->article_type;
        $data = [];

        if($articleType == 1)
            $data = News::select("id","title","image")->where("id",$dataId)->first();
        if($articleType == 2)
            $data = Poll::select("id","title","image")->where("id",$dataId)->first();
        if($articleType == 3)
            $data = Insight::select("id","title",DB::raw("'' as image"))->where("id",$dataId)->first();
        if($articleType == 4)
            $data = Ads::select(DB::raw("'' as title"),"image")->where("id",$dataId)->first();

        if(!empty($data)){
            $notification = [
                "title"=>$data->title,
                "image"=>$data->image
            ];

            $tokens = array_map(function($v){return $v['fcm_token'];},User::get("fcm_token")->toArray());
            $this->send_fcm("Android",$tokens,$notification,["data_id"=>$data->id,"article_type"=>$articleType, "title"=>$data->title,"image"=>$data->image]);

            $tokens = array_map(function($v){return $v['fcm_token_ios'];},User::get("fcm_token_ios")->toArray());
            $this->send_fcm("IOS",$tokens,$notification,["data_id"=>$data->id,"article_type"=>$articleType, "title"=>$data->title,"image"=>$data->image]);

            return ['status'=>true,"msg"=>"notification sended"];
        }else{
            return ['status'=>false,"msg"=>"data not found"];
        }

    }


    function notification_news_data(Request $request){
        if(empty($request->data_id) && empty($request->article_type))
            return response()->json(['success' => false, 'message' => 'Invalid Input Provided.']);

        try{

            $articleType = $request->article_type;
            $dataId = $request->data_id;
            if($articleType == 1){
                $data = News::where("id",$dataId)->first();
                $data->categories_name = NewsCategory::where("id",$data->news_category_id)->value("name");
                $data->is_bookmark = !empty(Bookmark::where("user_id",auth()->id())->where("news_id",$data->id)->first()) > 0 ? 1 : 0;
            }
            if($articleType == 2){
                $data = Poll::where("id",$dataId)->first();
                $data->categories_name = NewsCategory::where("id",$data->news_category_id)->value("name");
                $data->is_bookmark = 0;
            }
            if($articleType == 3){
                $data = Insight::with("insightImage")->where("id",$dataId)->first();
                $data->categories_name = NewsCategory::where("id",$data->news_category_id)->value("name") ?? '';
                $data->is_bookmark = 0;
            }
            if($articleType == 4){
                $data = Ads::where("id",$dataId)->first();
                $data->categories_name = "";
                $data->is_bookmark = 0;
            }

            return response()->json(['success' => true, 'data' => $data]);

        }catch (\Exception $e) {

            return response()->json(['success' => false, 'message' => $e->getMessage()]);
        }
    }
}
