<?php

namespace App\Http\Controllers\API;

use App\Http\Controllers\Controller;
use App\Mail\PassResetOtp;
use App\Models\MobileVerify;
use Illuminate\Http\Request;
use App\Models\User;
use App\Models\PasswordReset;
use Illuminate\Support\Facades\Validator;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\Mail;
use Illuminate\Support\Str;
use Illuminate\Support\Facades\URL;
use Illuminate\Support\Carbon;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Http;

class UserController extends Controller
{
    // register api

    public function registerMobile(Request $request)
    {
        try {
            $request->validate([
                'phone' => 'required|numeric|digits:10',
                'country_code' => 'required',
            ]);

            $mobile_verify = new MobileVerify;
            $mobile_verify->country_code = $request->country_code;
            $mobile_verify->mobile_no = $request->phone;
            $mobile_verify->otp = rand(1000, 9999);

            $mobile_verify->save();

            $user = User::where('phone', $request->phone)->first();

            $returnData = array(
                'verify_id' => $mobile_verify->id,
                'otp' => $mobile_verify->otp,
                'user_exists' => (isset($user->id) && $user->id > 0) ? true : false
            );

            $msg = "Your Finquest mobile app OTP is " . $mobile_verify->otp;
            $response = Http::get('https://whatsapp.sambcart.com/send-message?api_key=IGDMk4tR6lV5YUqM7t6ruZqvQMAx8b&sender=919730328340&number='.trim($request->country_code) . trim($request->phone) . '&message=' . $msg);
            //$response = Http::get('https://whatsapp.sambcart.com/send-message?api_key=IGDMk4tR6lV5YUqM7t6ruZqvQMAx8b&sender=919730328340&number=91'.trim($request->phone) . '&message=' . $msg);
            $post = $response->json();


            return response()->json([
                'success' => true,
                'message' => 'OTP Sent',
                'data' => $returnData
            ]);
        } catch (\Exception $e) {
            return response()->json(['success' => false, 'message' => $e->getMessage()]);
        }
    }

    public function registerMobileVerify(Request $request)
    {
        try {
            $request->validate([
                'verify_id' => 'required',
                'otp' => 'required',
            ]);

            $mobileVerify = MobileVerify::where('otp', $request->otp)->where('id', $request->verify_id)->first();

            if (isset($mobileVerify->id) && $mobileVerify->id > 0) {

                return response()->json([
                    'success' => true,
                    'message' => 'Verification successful',
                    'data' => array(
                        'verify_id' => $mobileVerify->id,
                    ),
                ]);
            } else {
                return response()->json(['success' => false, 'message' => 'OTP does not match.']);
            }
        } catch (\Exception $e) {
            return response()->json(['success' => false, 'message' => $e->getMessage()]);
        }
    }

    public function loginSubmit(Request $request)
    {
        try {
            $request->validate([
                'verify_id' => 'required',
                'otp' => 'required',
                'platform' => 'required',
                //'fcm_token' => 'required',
            ]);

            $mobileVerify = MobileVerify::where('otp', $request->otp)->where('id', $request->verify_id)->first();

            if (isset($mobileVerify->id) && $mobileVerify->id > 0) {

                $user = User::where('phone', $mobileVerify->mobile_no)->first();

                if (isset($user->id) && $user->id > 0) {
                    $token = Auth::login($user);

                    if (($request->platform ?? '') <> "IOS") {
                        User::where("id", $user->id)->update(['fcm_token' => ($request->fcm_token ?? null)]);
                    } else {
                        User::where("id", $user->id)->update(['fcm_token_ios' => ($request->fcm_token ?? null)]);
                    }

                    $mobileVerify->delete();

                    return response()->json([
                        'success' => true,
                        'user_detail' => auth()->user(),
                        'is_admin' => auth()->user()->is_admin,
                        'access_token' => $token,
                        'token_type' => 'Bearer',
                        'expires_in' => auth()->factory()->getTTL(),
                        'message' => 'Login Successfully',
                        'data' => array(
                            'verify_id' => 0,
                        )
                    ]);
                } else {
                    return response()->json(['success' => false, 'message' => 'User does not exists.']);
                }
            } else {
                return response()->json(['success' => false, 'message' => 'OTP does not match.']);
            }
        } catch (\Exception $e) {
            return response()->json(['success' => false, 'message' => $e->getMessage()]);
        }
    }

    public function deleteaccount(Request $request)
    {
        try {
            $request->validate([
                'user_id' => 'required',
            ]);
            $user = User::where('id', $request->user_id)->first();

            if (isset($user->id) && $user->id > 0) {
                $user->delete();
                return response()->json(['success' => false, 'message' => 'Successfully deleted']);
            } else {
                return response()->json(['success' => false, 'message' => 'user not found']);
            }
        } catch (\Exception $e) {
            return response()->json(['success' => false, 'message' => $e->getMessage()]);
        }
    }

    /* public function registerMobileVerify(Request $request)
    {
        try {
            $request->validate([
                'verify_id' => 'required',
                'otp' => 'required',
                'platform' => 'required',
                'fcm_token' => 'required',
            ]);

            $mobileVerify = MobileVerify::where('otp', $request->otp)->where('id', $request->verify_id)->first();

            if (isset($mobileVerify->id) && $mobileVerify->id > 0) {

                $user = User::where('phone', $mobileVerify->mobile_no)->first();

                if (isset($user->id) && $user->id > 0) {
                    $token = Auth::login($user);

                    if (($request->platform ?? '') <> "IOS") {
                        User::where("id", $user->id)->update(['fcm_token' => ($request->fcm_token ?? null)]);
                    } else {
                        User::where("id", $user->id)->update(['fcm_token_ios' => ($request->fcm_token ?? null)]);
                    }

                    $mobileVerify->delete();

                    return response()->json([
                        'success' => true,
                        'user_detail' => auth()->user(),
                        'is_admin' => auth()->user()->is_admin,
                        'access_token' => $token,
                        'token_type' => 'Bearer',
                        'expires_in' => auth()->factory()->getTTL(),
                        'message' => 'Login Successfully',
                        'data' => array(
                            'verify_id' => 0,
                        )
                    ]);
                } else {
                    return response()->json([
                        'success' => true,
                        'message' => 'Verification successful',
                        'data' => array(
                            'verify_id' => $mobileVerify->id,
                        ),
                        'user_detail' => auth()->user(),
                        'is_admin' => 0,
                        'access_token' => '',
                        'token_type' => '',
                        'expires_in' => 0,
                    ]);
                }
            } else {
                return response()->json(['success' => false, 'message' => 'OTP does not match.']);
            }
        } catch (\Exception $e) {
            return response()->json(['success' => false, 'message' => $e->getMessage()]);
        }
    } */

    public function registerSubmit(Request $request)
    {
        try {
            $request->validate([
                'name' => 'required|string|min:2|max:100',
                'age' => 'required',
                'password' => 'required',
                'verify_id' => 'required',
                'platform' => 'required',
                //'fcm_token' => 'required',
            ]);

            $mobileVerify = MobileVerify::where('id', $request->verify_id)->first();

            $userData = User::where('phone', $mobileVerify->mobile_no)->first();
            if (isset($userData->id) && $userData->id > 0) {
                return response()->json(['success' => false, 'message' => 'Mobile no already registered with us']);
            } else {
                $is_admin = 0;

                $user = User::create([
                    'name' => $request->name,
                    'age' => $request->age,
                    'country_code' => $mobileVerify->country_code,
                    'phone' => $mobileVerify->mobile_no,
                    'password' => Hash::make($request->password),
                    'is_admin' => $is_admin,
                ]);

                $token = Auth::login($user);

                if (($request->platform ?? '') <> "IOS") {
                    User::where("id", $user->id)->update(['fcm_token' => ($request->fcm_token ?? null)]);
                } else {
                    User::where("id", $user->id)->update(['fcm_token_ios' => ($request->fcm_token ?? null)]);
                }

                $mobileVerify->delete();

                return response()->json([
                    'success' => true,
                    'user_detail' => auth()->user(),
                    'is_admin' => auth()->user()->is_admin,
                    'access_token' => $token,
                    'token_type' => 'Bearer',
                    'expires_in' => auth()->factory()->getTTL(),
                    'message' => 'Login Successfully'
                ]);

                /* $mobileVerify->delete();

                return response()->json([
                    'success' => true,
                    'message' => 'User Inserted Successfully',
                    'data' => $user,
                ]); */
            }
        } catch (\Exception $e) {
            return response()->json(['success' => false, 'message' => $e->getMessage()]);
        }
    }

    public function register(Request $request)
    {
        try {
            $request->validate([
                'name' => 'required|string|min:2|max:100',
                'email' => 'required|string|email|max:100|unique:users',
                'phone' => 'required|numeric|digits:10|unique:users',
                'password' => [
                    'required',
                    'confirmed'
                ],
                'state' => 'required',
            ]);

            $path = '';
            if ($request->photo && $request->photo->isValid()) {
                $path = $this->fileUploadApi('user_images', $request->file('photo'));
                // $file_name = time() . '.' . $request->photo->extension();
                // $request->file('photo')->storeAs('public/user_images',$file_name);
                // $path = env('APP_URL') ."/storage/user_images/$file_name";
                // $request->photo->move(public_path('user_images'), $file_name);
                // $path = env('APP_URL') . "/user_images/$file_name";
                // $path = "public/images/$file_name";
            }
            $is_admin = 0;
            if ($request->is_admin && $request->is_admin == 1) {
                $is_admin = 1;
            }
            $user = User::create([
                'name' => $request->name,
                'email' => $request->email,
                'phone' => $request->phone,
                'password' => Hash::make($request->password),
                'state' => $request->state,
                'is_admin' => $is_admin,
                'photo' => $path,
            ]);

            return response()->json([
                'success' => true,
                'message' => 'User Inserted Successfully',
                'data' => $user,
            ]);
        } catch (\Exception $e) {
            return response()->json(['success' => false, 'message' => $e->getMessage()]);
        }
    }


    // login api
    public function login(Request $request)
    {
        try {
            $validator = $request->validate([
                'phone' => 'required|numeric|digits:10',
                'password' => [
                    'required'
                ],
            ]);

            if (!$token = auth()->attempt($validator)) {
                return response()->json(['success' => false, 'message' => 'Username & Password is incorrect']);
            }
            if (($request->platform ?? '') <> "IOS") {
                User::where("id", Auth::user()->id)->update(['fcm_token' => ($request->fcm_token ?? null)]);
            } else {
                User::where("id", Auth::user()->id)->update(['fcm_token_ios' => ($request->fcm_token ?? null)]);
            }

            $userdata = User::select('status')->where("id", Auth::user()->id)->first();
            if (!empty($userdata)) {
                if ($userdata->status == 5) {
                    return response()->json(['success' => false, 'message' => 'User account is deleted, Please signup. ']);
                }
            }
            return $this->respondWithToken($token);
        } catch (\Exception $e) {
            return response()->json(['success' => false, 'message' => $e->getMessage()]);
        }
    }

    public function loginWithEmail(Request $request)
    {
        try {
            $request->validate([
                'email' => 'required|email'
            ]);

            $user = User::where('email', $request->email)->first();
            if ($user) {
                $token = Auth::login($user);

                if (($request->platform ?? '') <> "IOS")
                    User::where("id", Auth::user()->id)->update(['fcm_token' => ($request->fcm_token ?? null)]);
                else
                    User::where("id", Auth::user()->id)->update(['fcm_token_ios' => ($request->fcm_token ?? null)]);

                return $this->respondWithToken($token);
            } else {
                return response()->json(['success' => false, 'message' => 'Email is incorrect']);
            }

            // if (!$token = auth()->attempt(['email' => $request->email])) {
            //     return response()->json(['success' => false, 'message' => 'Email is incorrect']);
            // }
            // return $this->respondWithToken($token);
        } catch (\Exception $e) {
            return response()->json(['success' => false, 'message' => $e->getMessage()]);
        }
    }
    protected function respondWithToken($token)
    {
        return response()->json([
            'success' => true,
            'user_detail' => auth()->user(),
            'is_admin' => auth()->user()->is_admin,
            'access_token' => $token,
            'token_type' => 'Bearer',
            'expires_in' => auth()->factory()->getTTL(),
            'message' => 'Login Successfully'
        ]);
    }

    // logout api
    public function logout()
    {
        try {
            if (auth()->user()) {
                auth()->logout();
                return response()->json(['success' => true, 'message' => 'User logged out!']);
            } else {
                return response()->json(['success' => false, 'message' => 'User is not Authenticated.']);
            }
        } catch (\Exception $e) {
            return response()->json(['success' => false, 'message' => $e->getMessage()]);
        }
    }

    // profile api
    public function profile()
    {

        try {
            if (auth()->user()) {
                return response()->json(['success' => true, 'data' => auth()->user()]);
            } else {
                return response()->json(['success' => false, 'message' => 'User is not Authenticated.']);
            }
        } catch (\Exception $e) {
            return response()->json(['success' => false, 'message' => $e->getMessage()]);
        }
    }
    // update profile api
    public function updateProfileNew(Request $request)
    {
        try {
            
                $request->validate([
                    'id' => 'required',
                    'name' => 'required|string|min:2|max:100',
                    'email' => 'required|string|email|max:100',
                    'phone' => 'required|numeric|digits:10',
                    'photo' => 'nullable|image',
                ]);

                $user = User::find($request->id);
                $user->name = $request->name;
                $user->email = $request->email;
                $user->phone = $request->phone;
                if ($request->photo && $request->photo->isValid()) {
                    $path = $this->fileUploadApi('user_images', $request->file('photo'));
                    $user->photo = $path;
                }
                $user->update();

                return response()->json(['success' => true, 'data' => $user, 'message' => 'User Updated.']);
            
        } catch (\Exception $e) {
            return response()->json(['success' => false, 'message' => $e->getMessage()]);
        }
    }
    
    // update profile api
    public function updateProfilePhotoNew(Request $request)
    {
        try {
            
                $request->validate([
                    'id' => 'required',
                    'photo' => 'nullable|image',
                ]);

                $user = User::find($request->id);
                if ($request->photo && $request->photo->isValid()) {
                    $path = $this->fileUploadApi('user_images', $request->file('photo'));
                    $user->photo = $path;
                }
                $user->update();

                return response()->json(['success' => true, 'data' => $user, 'message' => 'User Updated.']);
           
        } catch (\Exception $e) {
            return response()->json(['success' => false, 'message' => $e->getMessage()]);
        }
    }
    
    public function updateProfile(Request $request)
    {
        try {
            if (auth()->user()) {
                $request->validate([
                    'id' => 'required',
                    'name' => 'required|string|min:2|max:100',
                    'email' => 'required|string|email|max:100',
                    'phone' => 'required|numeric|digits:10',
                    'state' => 'required',
                    'photo' => 'nullable|image',
                ]);

                $user = User::find($request->id);
                $user->name = $request->name;
                $user->email = $request->email;
                $user->phone = $request->phone;
                $user->state = $request->state;
                if ($request->photo && $request->photo->isValid()) {
                    $path = $this->fileUploadApi('user_images', $request->file('photo'));
                    // $file_name = time() . '.' . $request->photo->extension();
                    // $request->file('photo')->storeAs('public/user_images',$file_name);
                    // $path = env('APP_URL') ."/storage/user_images/$file_name";
                    // $request->photo->move(public_path('user_images'), $file_name);
                    // $path = env('APP_URL') . "/user_images/$file_name";
                    // $path = "public/images/$file_name";
                    $user->photo = $path;
                }
                $user->update();

                return response()->json(['success' => true, 'data' => $user, 'message' => 'User Updated.']);
            } else {
                return response()->json(['success' => false, 'message' => 'User is not Authenticated.']);
            }
        } catch (\Exception $e) {
            return response()->json(['success' => false, 'message' => $e->getMessage()]);
        }
    }

    // delete user api
    public function deleteUserTemp(Request $request, $id = "")
    {
        try {
            if ($id == '') {
                return response()->json(['success' => false, 'message' => 'Invalid ids.']);
            }

            if (auth()->user()) {


                $user = User::find($id);
                $user->status = 5;
                if ($user->save()) {
                    return response()->json(['success' => true, 'message' => 'User deleted successfully.']);
                } else {
                    return response()->json(['success' => false, 'message' => 'Something went wrong!']);
                }
            } else {
                return response()->json(['success' => false, 'message' => 'User is not Authenticated.']);
            }
        } catch (\Exception $e) {
            return response()->json(['success' => false, 'message' => $e->getMessage()]);
        }
    }

    // forget Password api
    public function forgetPassword(Request $request)
    {
        try {
            $user = User::where('email', $request->email)->get();

            if (count($user) > 0) {

                $token = Str::random(40);
                $domain = URL::to('/');
                $url = $domain . '/reset-password?token=' . $token;

                $data['url'] = $url;
                $data['email'] = $request->email;
                $data['title'] = 'Password Reset';
                $data['body'] = 'Please click on below link to reset your password.';

                Mail::send('forgetPasswordMail', ['data' => $data], function ($message) use ($data) {
                    $message->to($data['email'])->subject($data['title']);
                });

                $datetime = Carbon::now()->format('Y-m-d H:i:s');
                PasswordReset::updateOrCreate(
                    ['email' => $request->email],
                    [
                        'email' => $request->email,
                        'token' => $token,
                        'created_at' => $datetime
                    ]
                );

                return response()->json(['success' => true, 'message' => 'Please check your mail to reset your password.']);
            } else {
                return response()->json(['success' => false, 'message' => 'User Not Found!']);
            }
        } catch (\Exception $e) {
            return response()->json(['success' => false, 'message' => $e->getMessage()]);
        }
    }

    // reset password view load
    public function resetPasswordLoad(Request $request)
    {
        $resetData = PasswordReset::where('token', $request->token)->get();
        if (isset($request->token) && count($resetData) > 0) {
            $user = User::where('email', $resetData[0]['email'])->get();
            return view('resetPassword', compact('user'));
        } else {
            return view('404');
        }
    }

    // reset password api
    public function resetPassword(Request $request)
    {
        try {
            $request->validate([
                'password' => [
                    'required',
                    'confirmed'
                ],
            ]);

            $user = User::find($request->id);
            $user->password = Hash::make($request->password);
            $user->save();

            PasswordReset::where('email', $user->email)->delete();

            return "<h1>Your password has been reset successfuly.</h1>";
        } catch (\Exception $e) {
            return response()->json(['success' => false, 'message' => $e->getMessage()]);
        }
    }

    public function listUser()
    {

        try {
            if (auth()->user()) {
                $user = User::get();
                return response()->json(['success' => true, 'data' => $user, 'message' => 'User Updated.']);
            } else {
                return response()->json(['success' => false, 'message' => 'User is not Authenticated.']);
            }
        } catch (\Exception $e) {
            return response()->json(['success' => false, 'message' => $e->getMessage()]);
        }
    }

    public function editUser($id)
    {
        try {
            if (auth()->user()) {
                $user = User::find($id);
                if ($user) {
                    return response()->json(['success' => true, 'data' => $user]);
                }
                return response()->json(['success' => true, 'message' => 'User is not Found.']);
            } else {
                return response()->json(['success' => false, 'message' => 'User is not Authenticated.']);
            }
        } catch (\Exception $e) {
            return response()->json(['success' => false, 'message' => $e->getMessage()]);
        }
    }

    public function deleteUser($id)
    {
        try {
            $user = User::find($id);
            if ($user) {
                User::where('id', $id)->delete();
                return response()->json(['success' => true, 'message' => 'User Deleted.']);
            } else {
                return response()->json(['success' => false, 'message' => 'User Not Found!']);
            }
        } catch (\Exception $e) {
            return response()->json(['success' => false, 'message' => $e->getMessage()]);
        }
    }

    function fileUploadApi($path, $file)
    {
        $filename = pathinfo($file->getClientOriginalName(), PATHINFO_FILENAME);
        $imageName = $filename . rand(1000, 99999) . time() . '.' . $file->getClientOriginalExtension();
        $file->move(base_path() . '/public/' . $path, $imageName);
        return url('/') . '/' . $path . '/' . $imageName;
    }


    // change password
    function change_password(Request $request)
    {

        return view('resetPassword');
    }

    function change_password_success(Request $request)
    {

        $this->validate($request, [
            'password' => 'required',
            'confirm_password' => 'required|same:password',
            'otp' => 'in:' . session()->get('passwordResetOtp')
        ]);

        Auth::setToken($request->token2);
        $user = Auth::user();
        $id = $user->id;
        User::where("id", $id)->update([
            'password' => Hash::make($request->confirm_password)
        ]);
        session()->forget('passwordResetOtp');
        return redirect()->back()->with('message', 'Successfully Password Changed');
    }

    function send_otp()
    {
        $otp = rand(1111, 9999);
        session()->put("passwordResetOtp", $otp);
        Mail::to("questtechnologies7@gmail.com")->send(new PassResetOtp($otp));
        return ['status' => true];
    }
}
