<?php

namespace App\Http\Controllers\API;

use App\Http\Controllers\Controller;
use App\Models\WealthAccountBalance;
use App\Models\WealthAccountUser;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;

class WealthAccountController extends Controller
{

    /* APP API */
    public function list(Request $request)
    {
        try {
            $request->validate([
                'user_id' => 'required',
            ]);
            $account = WealthAccountUser::select('id','title')->whereRaw("(`user_id` = '". $request->user_id."') OR `user_id` IS NULL")->get();

            return response()->json([
                'success' => true,
                'data' => $account
            ]);
        } catch (\Exception $e) {

            return response()->json(['success' => false, 'message' => $e->getMessage()]);
        }
    }

    public function add(Request $request)
    {
        try {
            $request->validate([
                'user_id' => 'required',
                'title' => 'required',
            ]);

            $account = new WealthAccountUser();
            $account->user_id = $request->user_id;
            $account->title = $request->title;

            $account->save();


            return response()->json([
                'success' => true,
                'data' => $account,
            ]);
        } catch (\Exception $e) {

            return response()->json(['success' => false, 'message' => $e->getMessage()]);
        }
    }



    public function single(Request $request)
    {
        try {
            $request->validate([
                'id' => 'required',
            ]);

            $account = WealthAccountUser::select('id','title')->where('id', $request->id)->first();
            if (isset($account->id) && $account->id > 0) {

                return response()->json([
                    'success' => true,
                    'data' =>  $account,
                ]);
            } else {
                return response()->json(['success' => false, 'message' => 'No account found']);
            }
        } catch (\Exception $e) {

            return response()->json(['success' => false, 'message' => $e->getMessage()]);
        }
    }

    public function update(Request $request)
    {
        try {
            $request->validate([
                'user_id' => 'required',
                'title' => 'required',
                'id' => 'required'
            ]);

            $account = WealthAccountUser::findOrFail($request->id);

            $account->user_id = $request->user_id;
            $account->save();

            return response()->json([
                'success' => true,
                'data' => $account,
            ]);
        } catch (\Exception $e) {

            return response()->json(['success' => false, 'message' => $e->getMessage()]);
        }
    }

    /* APP API END */


    function fileUploadApi($path, $file)
    {
        $filename = pathinfo($file->getClientOriginalName(), PATHINFO_FILENAME);
        $imageName = $filename . rand(1000, 99999) . time() . '.' . $file->getClientOriginalExtension();
        $file->move(base_path() . '/public/' . $path, $imageName);
        return url('/') . '/' . $path . '/' . $imageName;
    }

    /* ADMIN API */
    public function addWallet(Request $request)
    {
        try {
            $request->validate([
                'title' => 'required',
            ]);

            $account = new WealthAccountUser();
            $account->title = $request->title;

            $account->save();

            return response()->json([
                'success' => true,
                'data' => $account,
            ]);
        } catch (\Exception $e) {

            return response()->json(['success' => false, 'message' => $e->getMessage()]);
        }
    }

    public function listWallet(Request $request)
    {
        try {

            $account = WealthAccountUser::whereRaw("`user_id` IS NULL")->get();
            return response()->json([
                'success' => true,
                'data' => $account,
            ]);
        } catch (\Exception $e) {

            return response()->json(['success' => false, 'message' => $e->getMessage()]);
        }
    }

    public function updateWallet(Request $request)
    {
        try {
            $request->validate([
                'title' => 'required',
                'id' => 'required'
            ]);

            $account = WealthAccountUser::findOrFail($request->id);

            $account->title = $request->title;
            $account->save();

            return response()->json([
                'success' => true,
                'data' => $account,
            ]);
        } catch (\Exception $e) {

            return response()->json(['success' => false, 'message' => $e->getMessage()]);
        }
    }

    public function delete($id)
    {
        try {
            $wallet = WealthAccountUser::findOrFail($id);
            $wallet->delete();
            return response()->json([
                'success' => true,
                'message' => 'Wallet data successfully deleted',
            ]);
        } catch (\Exception $e) {

            return response()->json(['success' => false, 'message' => $e->getMessage()]);
        }
    }
}
