<?php

namespace App\Http\Controllers\API;

use App\Http\Controllers\Controller;
use App\Models\WealthCategory;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;

class WealthCategoryController extends Controller
{

    /* APP API */
    public function add(Request $request)
    {
        try {
            $request->validate([
                'user_id' => 'required',
                'title' => 'required',
                'icon' => 'required',
                'color' => 'required',
                'category_type' => 'required',
            ]);

            $category = new WealthCategory();
            $category->user_id = $request->user_id;
            $category->title = $request->title;
            $category->color = $request->color;
            $category->icon = $request->icon;
            $category->category_type = strtoupper($request->category_type);

            $category->save();


            return response()->json([
                'success' => true,
                'data' => $category,
            ]);
        } catch (\Exception $e) {

            return response()->json(['success' => false, 'message' => $e->getMessage()]);
        }
    }


    public function single(Request $request)
    {
        try {
            $request->validate([
                'id' => 'required'
            ]);

            $category = WealthCategory::select('id','title','icon','color','category_type')->where('id', $request->id)->first();

            if (isset($category->id) && $category->id > 0) {

                return response()->json([
                    'success' => true,
                    'data' => $category,
                ]);
            } else {
                return response()->json(['success' => false, 'message' => 'No category found']);
            }
        } catch (\Exception $e) {

            return response()->json(['success' => false, 'message' => $e->getMessage()]);
        }
    }

    public function update(Request $request)
    {
        try {
            $request->validate([
                'user_id' => 'required',
                'title' => 'required',
                'color' => 'required',
                'category_type' => 'required',
                'id' => 'required'
            ]);

            $category = WealthCategory::findOrFail($request->id);

            $category->user_id = $request->user_id;
            $category->title = $request->title;
            $category->color = $request->color;
            $category->icon = $request->icon;
            $category->category_type = strtoupper($request->category_type);

            $category->save();


            return response()->json([
                'success' => true,
                'data' => $category,
            ]);
        } catch (\Exception $e) {

            return response()->json(['success' => false, 'message' => $e->getMessage()]);
        }
    }

    public function list(Request $request)
    {
        try {
            $request->validate([
                'user_id' => 'required',
                'category_type' => 'required',
            ]);

            $category = WealthCategory::select('id','title','icon','color','category_type')->whereRaw("(`user_id` = ? OR `user_id` IS NULL) AND `category_type` = ?", array($request->user_id, strtoupper($request->category_type)))
                ->get();

            if (isset($category) && count($category) > 0) {
                return response()->json([
                    'success' => true,
                    'data' => $category,
                ]);
            } else {
                return response()->json(['success' => false, 'message' => 'No data found']);
            }
        } catch (\Exception $e) {

            return response()->json(['success' => false, 'message' => $e->getMessage()]);
        }
    }

    public function delete($id)
    {
        try {
            $category = WealthCategory::findOrFail($id);
            $category->delete();
            return response()->json([
                'success' => true,
                'message' => 'category data successfully deleted',
            ]);
        } catch (\Exception $e) {

            return response()->json(['success' => false, 'message' => $e->getMessage()]);
        }
    }

    /* ADMIN API */

    public function addCategory(Request $request)
    {
        try {
            $request->validate([
                'title' => 'required',
                'icon' => 'required',
                'color' => 'required',
                'category_type' => 'required',
            ]);

            $path = '';
            if ($request->icon && $request->icon->isValid()) {
                $path = $this->fileUploadApi('category_icon', $request->file('icon'));
            }

            $category = new WealthCategory();
            $category->title = $request->title;
            $category->color = $request->color;
            $category->icon = $path;
            $category->category_type = strtoupper($request->category_type);

            $category->save();


            return response()->json([
                'success' => true,
                'data' => $category,
            ]);
        } catch (\Exception $e) {

            return response()->json(['success' => false, 'message' => $e->getMessage()]);
        }
    }

    public function updateCategory(Request $request)
    {
        try {
            $request->validate([
                'title' => 'required',
                'color' => 'required',
                'category_type' => 'required',
                'id' => 'required'
            ]);

            $category = WealthCategory::findOrFail($request->id);

            $path = '';
            if ($request->icon && $request->icon->isValid()) {
                $path = $this->fileUploadApi('category_icon', $request->file('icon'));
                $category->icon = $path;
            }

            $category->title = $request->title;
            $category->color = $request->color;
            $category->category_type = strtoupper($request->category_type);

            $category->save();


            return response()->json([
                'success' => true,
                'data' => $category,
            ]);
        } catch (\Exception $e) {

            return response()->json(['success' => false, 'message' => $e->getMessage()]);
        }
    }

    function fileUploadApi($path, $file)
    {
        $filename = pathinfo($file->getClientOriginalName(), PATHINFO_FILENAME);
        $imageName = $filename . rand(1000, 99999) . time() . '.' . $file->getClientOriginalExtension();
        $file->move(base_path() . '/public/' . $path, $imageName);
        return url('/') . '/' . $path . '/' . $imageName;
    }
}
