<?php

namespace App\Http\Controllers\API;

use App\Http\Controllers\Controller;
use App\Models\WealthSplitEvent;
use App\Models\WealthSplitMember;
use App\Models\WealthSplitTransaction;
use App\Models\WealthSplitTransactionMember;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;

class WealthSplitEventMemberController extends Controller
{

    /* APP API */
    public function add(Request $request)
    {
        try {
            $request->validate([
                'event_id' => 'required',
                'name' => 'required',
            ]);

            $event = WealthSplitEvent::where('id', $request->event_id)->first();

            if (isset($event->id) && $event->id > 0) {

                $member = new WealthSplitMember();
                $member->event_id = $request->event_id;
                $member->name = $request->name;

                $member->save();


                return response()->json([
                    'success' => true,
                    'data' => $member,
                ]);
            } else {
                return response()->json(['success' => false, 'message' => 'Sorry! Event does not exists.']);
            }
        } catch (\Exception $e) {

            return response()->json(['success' => false, 'message' => $e->getMessage()]);
        }
    }

    public function delete(Request $request)
    {
        try {
            $request->validate([
                'member_id' => 'required',
            ]);

            $transaction = WealthSplitTransactionMember::where('payer_id', $request->member_id)->first();
            if (isset($transaction->id) && $transaction->id > 0) {
                return response()->json([
                    'success' => false,
                    'message' => 'This member is related in one of your past transaction! Please delete the transaction first.',
                ]);
            } else {
                WealthSplitTransactionMember::where('member_id', $request->member_id)->delete();

                $event = WealthSplitMember::findOrFail($request->member_id);
                $event->delete();
                return response()->json([
                    'success' => true,
                    'message' => 'member data successfully deleted',
                ]);
            }
        } catch (\Exception $e) {

            return response()->json(['success' => false, 'message' => $e->getMessage()]);
        }
    }

    public function list(Request $request)
    {
        try {
            $request->validate([
                'event_id' => 'required',
                'user_id' => 'required',
            ]);

            $i = 0;
            $member = WealthSplitMember::where('event_id', $request->event_id)->get();
            if (isset($member) && count($member) > 0) {
                foreach ($member as $key => $val) {
                    if ($val->user_id == $request->user_id) {
                        $memberData[$i] = array(
                            'id' => $val->id,
                            'event_id' => $val->event_id,
                            'name' => $val->name,
                            'user_id' => ($val->user_id > 0) ? $val->user_id : 0,
                            'login_user' => ($val->user_id == $request->user_id) ? 'YES' : 'NO',
                        );

                        $i++;
                    }
                }
                foreach ($member as $key => $val) {
                    if ($val->user_id != $request->user_id) {
                        $memberData[$i] = array(
                            'id' => $val->id,
                            'event_id' => $val->event_id,
                            'name' => $val->name,
                            'user_id' => ($val->user_id > 0) ? $val->user_id : 0,
                            'login_user' => ($val->user_id == $request->user_id) ? 'YES' : 'NO',
                        );

                        $i++;
                    }
                }


                return response()->json([
                    'success' => true,
                    'data' => $memberData,
                ]);
            } else {
                return response()->json(['success' => false, 'message' => 'No data found']);
            }
        } catch (\Exception $e) {

            return response()->json(['success' => false, 'message' => $e->getMessage()]);
        }
    }

    /* ADMIN API */

    public function addCategory(Request $request)
    {
        try {
            $request->validate([
                'title' => 'required',
                'icon' => 'required',
                'color' => 'required',
                'member_type' => 'required',
            ]);

            $path = '';
            if ($request->icon && $request->icon->isValid()) {
                $path = $this->fileUploadApi('member_icon', $request->file('icon'));
            }

            $member = new WealthSplitMember();
            $member->title = $request->title;
            $member->color = $request->color;
            $member->icon = $path;
            $member->member_type = strtoupper($request->member_type);

            $member->save();


            return response()->json([
                'success' => true,
                'data' => $member,
            ]);
        } catch (\Exception $e) {

            return response()->json(['success' => false, 'message' => $e->getMessage()]);
        }
    }

    public function updateCategory(Request $request)
    {
        try {
            $request->validate([
                'title' => 'required',
                'color' => 'required',
                'member_type' => 'required',
                'id' => 'required'
            ]);

            $member = WealthSplitMember::findOrFail($request->id);

            $path = '';
            if ($request->icon && $request->icon->isValid()) {
                $path = $this->fileUploadApi('member_icon', $request->file('icon'));
                $member->icon = $path;
            }

            $member->title = $request->title;
            $member->color = $request->color;
            $member->member_type = strtoupper($request->member_type);

            $member->save();


            return response()->json([
                'success' => true,
                'data' => $member,
            ]);
        } catch (\Exception $e) {

            return response()->json(['success' => false, 'message' => $e->getMessage()]);
        }
    }

    function fileUploadApi($path, $file)
    {
        $filename = pathinfo($file->getClientOriginalName(), PATHINFO_FILENAME);
        $imageName = $filename . rand(1000, 99999) . time() . '.' . $file->getClientOriginalExtension();
        $file->move(base_path() . '/public/' . $path, $imageName);
        return url('/') . '/' . $path . '/' . $imageName;
    }
}
