<?php

namespace App\Http\Controllers\API;

use App\Http\Controllers\Controller;
use App\Models\User;
use App\Models\WealthSplitEvent;
use App\Models\WealthSplitMember;
use App\Models\WealthSplitTransaction;
use App\Models\WealthSplitTransactionMember;
use PDF;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;

class WealthSplitTransactionController extends Controller
{

    /* APP API */
    public function add(Request $request)
    {
        try {
            $request->validate([
                'event_id' => 'required',
                'category_id' => 'required',
                'account_id' => 'required',
                'payer_id' => 'required',
                'amount' => 'required',
                'transaction_date' => 'required',
                'transaction_type' => 'required',
                'split_member_id' => 'required',
            ]);

            $transaction = new WealthSplitTransaction();
            $transaction->event_id = $request->event_id;
            $transaction->category_id = $request->category_id;
            $transaction->account_id = $request->account_id;
            $transaction->payer_id = $request->payer_id;
            $transaction->amount = $request->amount;
            $transaction->comment = $request->comment;
            $transaction->transaction_type = $request->transaction_type;
            $transaction->transaction_date = $request->transaction_date . ' ' . date('H:i:s');
            $transaction->save();
            $id = $transaction->id;

            $splitBetween = $request->split_member_id;

            if (isset($splitBetween) && !empty($splitBetween)) {
                $countSplit = count($splitBetween);
                $splitAmt = round(($request->amount / $countSplit), 3);

                foreach ($splitBetween as $key => $val) {
                    $split = new WealthSplitTransactionMember();
                    $split->transaction_id = $id;
                    $split->member_id = $val;
                    $split->payer_id = $transaction->payer_id;
                    $split->transaction_type = $request->transaction_type;
                    $split->amount = ($val != $transaction->payer_id) ? (($request->transaction_type == 'INCOME') ? -$splitAmt : $splitAmt) : 0;
                    $split->save();
                }
            }

            return response()->json([
                'success' => true,
                'data' => $transaction,
            ]);
        } catch (\Exception $e) {

            return response()->json(['success' => false, 'message' => $e->getMessage()]);
        }
    }

    public function update(Request $request)
    {
        try {
            $request->validate([
                'id' => 'required',
                'event_id' => 'required',
                'category_id' => 'required',
                'account_id' => 'required',
                'payer_id' => 'required',
                'amount' => 'required',
                'transaction_date' => 'required',
                'transaction_type' => 'required',
                'split_member_id' => 'required'
            ]);

            $checkExists = WealthSplitTransaction::where('id', $request->id)->first();

            if (isset($checkExists->id) && $checkExists->id > 0) {

                $transaction = WealthSplitTransaction::findOrFail($request->id);
                $transaction->event_id = $request->event_id;
                $transaction->category_id = $request->category_id;
                $transaction->account_id = $request->account_id;
                $transaction->payer_id = $request->payer_id;
                $transaction->amount = $request->amount;
                $transaction->transaction_type = $request->transaction_type;
                $transaction->comment = $request->comment;
                $transaction->transaction_date = $request->transaction_date . ' ' . date('H:i:s');
                $transaction->save();

                $id = $request->id;
                $splitBetween = $request->split_member_id;

                if (isset($splitBetween) && !empty($splitBetween)) {
                    $countSplit = count($splitBetween);
                    $splitAmt = round(($request->amount / $countSplit), 3);

                    WealthSplitTransactionMember::where('transaction_id', $id)->delete();

                    foreach ($splitBetween as $key => $val) {
                        $split = new WealthSplitTransactionMember();
                        $split->transaction_id = $id;
                        $split->member_id = $val;
                        $split->payer_id = $transaction->payer_id;
                        $split->transaction_type = $request->transaction_type;
                        $split->amount = ($val != $transaction->payer_id) ? (($request->transaction_type == 'INCOME') ? -$splitAmt : $splitAmt) : 0;
                        $split->save();
                    }
                }

                return response()->json([
                    'success' => true,
                    'data' => $transaction,
                ]);
            } else {
                return response()->json(['success' => false, 'message' => 'Sorry! No data found.']);
            }
        } catch (\Exception $e) {

            return response()->json(['success' => false, 'message' => $e->getMessage()]);
        }
    }

    public function delete(Request $request)
    {
        try {
            $request->validate([
                'transaction_id' => 'required',
            ]);
            WealthSplitTransactionMember::where('transaction_id', $request->transaction_id)->delete();

            $oldTrans = WealthSplitTransaction::findOrFail($request->transaction_id);

            $oldTrans->delete();

            return response()->json(['success' => true, 'message' => 'successfully deleted']);
        } catch (\Exception $e) {

            return response()->json(['success' => false, 'message' => $e->getMessage()]);
        }
    }

    public function list(Request $request)
    {
        try {
            $request->validate([
                'event_id' => 'required',
            ]);
            $data = array();

            $event = WealthSplitEvent::with('member')->where('id', $request->event_id)->first();

            if (isset($event->id) && $event->id > 0) {

                $data['event']['id'] = $event->id;
                $data['event']['name'] = $event->name;
                if (isset($event->member) && !empty($event->member)) {
                    foreach ($event->member as $key => $val) {
                        $data['event']['member'][$key]['member_id'] = $val->id;
                        $data['event']['member'][$key]['name'] = $val->name;
                    }
                }

                $transaction = WealthSplitTransaction::with('category')
                    ->with('account')
                    ->with('payer')
                    ->with('member')
                    ->where('event_id', $event->id)
                    ->orderBy('transaction_date', 'desc')
                    ->get();


                $recordData = array();
                $i = 0;
                $k = 0;
                if (isset($transaction) && count($transaction) > 0) {
                    foreach ($transaction as $key => $val) {

                        $transaction_date = date('d-m-Y', strtotime($val->transaction_date));
                        if (isset($recordData) && !empty($recordData)) {
                            if (array_search($transaction_date, array_column($recordData, 'date')) !== FALSE) {
                                $recordData[$i]['total_amount'] = (isset($recordData[$i]['total_amount'])) ? $recordData[$i]['total_amount'] + (($val->transaction_type == 'INCOME') ? $val->amount : -$val->amount) : (($val->transaction_type == 'INCOME') ? $val->amount : -$val->amount);
                                $recordData[$i]['data'][$k]['category'] = $val->category->title;
                                $recordData[$i]['data'][$k]['category'] = $val->category->title;
                                $recordData[$i]['data'][$k]['icon'] = $val->category->icon;
                                $recordData[$i]['data'][$k]['color'] = $val->category->color;
                                $recordData[$i]['data'][$k]['transaction_id'] = $val->id;
                                $recordData[$i]['data'][$k]['account'] = $val->account->title;
                                $recordData[$i]['data'][$k]['amount'] = $val->amount;
                                $recordData[$i]['data'][$k]['note'] = $val->comment;
                                $recordData[$i]['data'][$k]['payer'] = $val->payer->name;
                                $recordData[$i]['data'][$k]['transaction_date'] = $val->transaction_date;
                                $recordData[$i]['data'][$k]['transaction_type'] = $val->transaction_type;

                                if (isset($val->member) && count($val->member) > 0) {
                                    foreach ($val->member as $ky => $v) {
                                        $splitBtwn = WealthSplitMember::where('id', $v->member_id)->first();
                                        $recordData[$i]['data'][$k]['splitbetween'][$ky] = $splitBtwn->name;
                                    }
                                }
                            } else {
                                $k = 0;
                                $i++;
                                $recordData[$i]['date'] = $transaction_date;
                                $recordData[$i]['total_amount'] = (isset($recordData[$i]['total_amount'])) ? $recordData[$i]['total_amount'] + (($val->transaction_type == 'INCOME') ? $val->amount : -$val->amount) : (($val->transaction_type == 'INCOME') ? $val->amount : -$val->amount);
                                $recordData[$i]['data'][$k]['category'] = $val->category->title;
                                $recordData[$i]['data'][$k]['category'] = $val->category->title;
                                $recordData[$i]['data'][$k]['icon'] = $val->category->icon;
                                $recordData[$i]['data'][$k]['color'] = $val->category->color;
                                $recordData[$i]['data'][$k]['transaction_id'] = $val->id;
                                $recordData[$i]['data'][$k]['account'] = $val->account->title;
                                $recordData[$i]['data'][$k]['amount'] = $val->amount;
                                $recordData[$i]['data'][$k]['note'] = $val->comment;
                                $recordData[$i]['data'][$k]['payer'] = $val->payer->name;
                                $recordData[$i]['data'][$k]['transaction_date'] = $val->transaction_date;
                                $recordData[$i]['data'][$k]['transaction_type'] = $val->transaction_type;

                                if (isset($val->member) && count($val->member) > 0) {
                                    foreach ($val->member as $ky => $v) {
                                        $splitBtwn = WealthSplitMember::where('id', $v->member_id)->first();
                                        $recordData[$i]['data'][$k]['splitbetween'][$ky] = $splitBtwn->name;
                                    }
                                }
                            }
                        } else {
                            $k = 0;
                            $recordData[$i]['date'] = $transaction_date;
                            $recordData[$i]['total_amount'] = (isset($recordData[$i]['total_amount'])) ? $recordData[$i]['total_amount'] + (($val->transaction_type == 'INCOME') ? $val->amount : -$val->amount) : (($val->transaction_type == 'INCOME') ? $val->amount : -$val->amount);
                            $recordData[$i]['data'][$k]['category'] = $val->category->title;
                            $recordData[$i]['data'][$k]['icon'] = $val->category->icon;
                            $recordData[$i]['data'][$k]['color'] = $val->category->color;
                            $recordData[$i]['data'][$k]['transaction_id'] = $val->id;
                            $recordData[$i]['data'][$k]['account'] = $val->account->title;
                            $recordData[$i]['data'][$k]['amount'] = $val->amount;
                            $recordData[$i]['data'][$k]['note'] = $val->comment;
                            $recordData[$i]['data'][$k]['payer'] = $val->payer->name;
                            $recordData[$i]['data'][$k]['transaction_date'] = $val->transaction_date;
                            $recordData[$i]['data'][$k]['transaction_type'] = $val->transaction_type;

                            if (isset($val->member) && count($val->member) > 0) {
                                foreach ($val->member as $ky => $v) {
                                    $splitBtwn = WealthSplitMember::where('id', $v->member_id)->first();
                                    $recordData[$i]['data'][$k]['splitbetween'][$ky] = $splitBtwn->name;
                                }
                            }
                        }
                        $k++;
                    }
                }
                $data['event']['transaction'] = $recordData;

                $totalIncome = WealthSplitTransaction::where('event_id', $event->id)
                    ->where('transaction_type', 'INCOME')
                    ->orderBy('transaction_date', 'desc')
                    ->sum('amount');

                $data['event']['total_income'] = $totalIncome;

                $totalExpense = WealthSplitTransaction::where('event_id', $event->id)
                    ->where('transaction_type', 'EXPENSE')
                    ->orderBy('transaction_date', 'desc')
                    ->sum('amount');

                $data['event']['total_expense'] = $totalExpense;
                $data['event']['balance'] = $totalIncome - $totalExpense;

                return response()->json([
                    'success' => true,
                    'data' => $data,
                ]);
            } else {
                return response()->json(['success' => false, 'message' => 'No event found']);
            }
        } catch (\Exception $e) {

            return response()->json(['success' => false, 'message' => $e->getMessage()]);
        }
    }

    public function singleTransaction(Request $request)
    {
        try {
            $request->validate([
                'transaction_id' => 'required',
            ]);

            $transaction = WealthSplitTransaction::with('category')
                ->with('account')
                ->with('payer')
                ->with('member')
                ->where('id', $request->transaction_id)
                ->first();

            if (isset($transaction->id) && $transaction->id > 0) {
                $recordData['category'] = $transaction->category->title;
                $recordData['icon'] = $transaction->category->icon;
                $recordData['color'] = $transaction->category->color;
                $recordData['transaction_id'] = $transaction->id;
                $recordData['account'] = $transaction->account->title;
                $recordData['amount'] = $transaction->amount;
                $recordData['note'] = $transaction->comment;
                $recordData['payer'] = $transaction->payer->name;
                $recordData['payer_id'] = $transaction->payer->id;
                $recordData['transaction_date'] = $transaction->transaction_date;
                $recordData['transaction_type'] = $transaction->transaction_type;

                if (isset($transaction->member) && count($transaction->member) > 0) {
                    foreach ($transaction->member as $ky => $v) {
                        $splitBtwn = WealthSplitMember::where('id', $v->member_id)->first();
                        $recordData['splitbetween'][$ky]['name'] = $splitBtwn->name;
                        $recordData['splitbetween'][$ky]['id'] = $splitBtwn->id;
                        $recordData['splitbetween'][$ky]['user_id'] = ($splitBtwn->user_id > 0) ? $splitBtwn->user_id : 0;
                    }
                }

                return response()->json([
                    'success' => true,
                    'data' => $recordData,
                ]);
            } else {
                return response()->json(['success' => false, 'message' => 'Sorry!! No transaction found.']);
            }
        } catch (\Exception $e) {

            return response()->json(['success' => false, 'message' => $e->getMessage()]);
        }
    }


    /* SPLIT FUNCTION */

    function getMin($arr, $N)
    {
        $minInd = 0;
        for ($i = 1; $i < $N; $i++) {
            if ($arr[$i] < $arr[$minInd]) {
                $minInd = $i;
            }
        }
        return $minInd;
    }

    function getMax($arr, $N)
    {
        $maxInd = 0;
        for ($i = 1; $i < $N; $i++) {
            if ($arr[$i] > $arr[$maxInd]) {
                $maxInd = $i;
            }
        }
        return $maxInd;
    }

    function minOf2($x, $y)
    {
        return $x < $y ? $x : $y;
    }

    protected $finalArray = array();

    function minCashFlowRec($amount, $N)
    {

        $mxCredit = $this->getMax($amount, $N);
        $mxDebit = $this->getMin($amount, $N);

        // If both amounts are 0, then
        // all amounts are settled
        if ($amount[$mxCredit] == 0 && $amount[$mxDebit] == 0) {
            return;
        }

        $min = $this->minOf2(-$amount[$mxDebit], $amount[$mxCredit]);
        $amount[$mxCredit] -= $min;
        $amount[$mxDebit] += $min;

        array_push($this->finalArray, [$mxDebit, $mxCredit, $min]);

        $this->minCashFlowRec($amount, $N);
    }

    function minCashFlow($graph, $N)
    {
        $amount = array_fill(0, $N, 0);
        for ($p = 0; $p < $N; $p++) {
            for ($i = 0; $i < $N; $i++) {
                $amount[$p] += $graph[$i][$p] - $graph[$p][$i];
            }
        }

        $this->minCashFlowRec($amount, $N);
    }

    /* SPLIT FUNCTION END */

    public function view(Request $request)
    {
        try {
            $request->validate([
                'event_id' => 'required',
            ]);

            $data = array();

            $event = WealthSplitEvent::with('member')->where('id', $request->event_id)->first();

            if (isset($event->id) && $event->id > 0) {

                $data['event']['id'] = $event->id;
                $data['event']['name'] = $event->name;
                $data['event']['created_at'] = date('d-m-Y H:i', strtotime($event->created_at));
                $data['event']['updated_at'] = date('d-m-Y H:i', strtotime($event->updated_at));
                if (isset($event->member) && !empty($event->member)) {
                    foreach ($event->member as $key => $val) {
                        $data['event']['member'][$key]['member_id'] = $val->id;
                        $data['event']['member'][$key]['name'] = $val->name;
                    }
                }

                $transaction = WealthSplitTransaction::with('category')
                    ->with('payer')
                    ->with('member')
                    ->where('event_id', $event->id)
                    ->orderBy('transaction_date', 'desc')
                    ->get();


                $recordData = array();
                $i = 0;
                $k = 0;
                if (isset($transaction) && count($transaction) > 0) {
                    foreach ($transaction as $key => $val) {
                        $recordData[$k]['category'] = $val->category->title;
                        $recordData[$k]['amount'] = $val->amount;
                        $recordData[$k]['note'] = $val->note;
                        $recordData[$k]['payer'] = $val->payer->name;
                        $recordData[$k]['transaction_date'] = $val->transaction_date;

                        if (isset($val->member) && count($val->member) > 0) {
                            foreach ($val->member as $ky => $v) {
                                $splitBtwn = WealthSplitMember::where('id', $v->member_id)->first();
                                $recordData[$k]['splitbetween'][$ky] = $splitBtwn->name;
                            }
                        }
                        $k++;
                    }
                }

                $data['event']['transaction'] = $recordData;

                /* $splitData = WealthSplitTransactionMember::with('payer')
                    ->with('member')
                    ->get(); */

                /* $splitData = DB::table('wealth_split_transaction_member as transaction')
                    ->leftJoin('wealth_split_transaction as tn', 'tn.id', '=', 'transaction.transaction_id')
                    ->leftJoin('wealth_split_member as member', 'member.id', '=', 'transaction.member_id')
                    ->leftJoin('wealth_split_member as payer', 'payer.id', '=', 'transaction.payer_id')
                    ->selectRaw(
                        'member.name as from_name,
                         member.id as from_id,
                         payer.name as to_name,
                         payer.id as to_id,
                         sum(transaction.amount) as amount,
                         transaction.transaction_type'
                    )
                    ->where('tn.event_id',$event->id)
                    ->groupBy('transaction.member_id', 'transaction.payer_id')
                    ->get();
                $data['event']['splitData'] = $splitData; */

                $eventMember = WealthSplitMember::where('event_id', $event->id)->orderBy('id', 'asc')->get();
                $data['event']['member'] = $eventMember;

                $graph = [];
                $member = [];
                if (isset($data['event']['member']) && count($data['event']['member']) > 0) {
                    foreach ($data['event']['member'] as $key => $val) {
                        $member[$key]['id'] = $val->id;
                        $member[$key]['name'] = $val->name;
                    }
                }
                for ($i = 0; $i < count($member); $i++) {
                    $newArr = array();
                    for ($k = 0; $k < count($member); $k++) {
                        $payer_id = $member[$i]['id'];
                        $member_id = $member[$k]['id'];

                        $splitTran = WealthSplitTransactionMember::where('member_id', $member_id)->where('payer_id', $payer_id)->sum('amount');

                        array_push($newArr, round($splitTran));
                    }

                    array_push($graph, $newArr);
                }
                $this->minCashFlow($graph, count($member));

                $splitDataFinal = $this->finalArray;

                $data['event']['splitDataFinal'] = $splitDataFinal;
                //expense data

                $expenseData = DB::table('wealth_split_transaction as transaction')
                    ->leftJoin('wealth_split_member as payer', 'payer.id', '=', 'transaction.payer_id')
                    ->selectRaw(
                        'payer.name as name,
                     sum(transaction.amount) as amount'
                    )
                    ->where('transaction.transaction_type', 'EXPENSE')
                    ->where('transaction.event_id', $event->id)
                    ->groupBy('transaction.payer_id')
                    ->get();
                $data['event']['expenseData'] = $expenseData;

                //income data
                $incomeData = DB::table('wealth_split_transaction as transaction')
                    ->leftJoin('wealth_split_member as payer', 'payer.id', '=', 'transaction.payer_id')
                    ->selectRaw(
                        'payer.name as name,
                     sum(transaction.amount) as amount'
                    )
                    ->where('transaction.transaction_type', 'INCOME')
                    ->where('transaction.event_id', $event->id)
                    ->groupBy('transaction.payer_id')
                    ->get();
                $data['event']['incomeData'] = $incomeData;



                $pdf = PDF::loadView('pdfs.split', $data);

                $path = base_path() . '/public/pdf/';
                $fileName = 'split_' . $event->id . '.pdf';
                $pdf->save($path  . $fileName);
                $url = url('/') . '/pdf/' . $fileName;

                return response()->json([
                    'success' => true,
                    'data' => $url,
                ]);
            } else {
                return response()->json(['success' => false, 'message' => 'No event found']);
            }
        } catch (\Exception $e) {

            return response()->json(['success' => false, 'message' => $e->getMessage()]);
        }
    }



    function fileUploadApi($path, $file)
    {
        $filename = pathinfo($file->getClientOriginalName(), PATHINFO_FILENAME);
        $imageName = $filename . rand(1000, 99999) . time() . '.' . $file->getClientOriginalExtension();
        $file->move(base_path() . '/public/' . $path, $imageName);
        return url('/') . '/' . $path . '/' . $imageName;
    }
}
