<?php

use Illuminate\Http\Request;
use Illuminate\Support\Facades\Route;
use App\Http\Controllers\API\UserController;
use App\Http\Controllers\API\NewsController;
use App\Http\Controllers\API\InsightController;
use App\Http\Controllers\API\PollController;
use App\Http\Controllers\API\HomeController;
use App\Http\Controllers\API\CourseController;
use App\Http\Controllers\API\AdsController;
use App\Http\Controllers\API\BannerController;
use App\Http\Controllers\API\MarketApiController;
use App\Http\Controllers\API\HoldingsController;
use App\Http\Controllers\API\HoldingsUserTypeController;
use App\Http\Controllers\API\WealthAccountController;
use App\Http\Controllers\API\WealthCategoryController;
use App\Http\Controllers\API\WealthColorController;
use App\Http\Controllers\API\WealthCurrencyController;
use App\Http\Controllers\API\WealthIconController;
use App\Http\Controllers\API\WealthSplitCategoryController;
use App\Http\Controllers\API\WealthSplitEventController;
use App\Http\Controllers\API\WealthSplitEventMemberController;
use App\Http\Controllers\API\WealthSplitTransactionController;
use App\Http\Controllers\API\WealthTransactionController;
use App\Http\Controllers\API\WealthUserCurrencyController;
use Illuminate\Support\Facades\Artisan;

/*
|--------------------------------------------------------------------------
| API Routes
|--------------------------------------------------------------------------
|
| Here is where you can register API routes for your application. These
| routes are loaded by the RouteServiceProvider within a group which
| is assigned the "api" middleware group. Enjoy building your API!
|
*/

Route::get('/migrate', function () {
    Artisan::call('migrate');
    dd('migrated!');
});

Route::post('/forget-password', [UserController::class, 'forgetPassword']);

Route::group(['middleware' => 'api'], function ($routes) {

    // User
    Route::get('/list-user', [UserController::class, 'listUser']);
    Route::get('/view-user/{id}', [UserController::class, 'viewUser']);
    Route::get('/edit-user/{id}', [UserController::class, 'editUser']);
    Route::post('/update-user', [UserController::class, 'updateUser']);
    Route::get('/delete-user/{id}', [UserController::class, 'deleteUser']);
    Route::get('/v2-delete-user/{id}', [UserController::class, 'deleteUserTemp']);

    // Auth
    Route::post('/register/mobile', [UserController::class, 'registerMobile']);
    Route::post('/register/verify', [UserController::class, 'registerMobileVerify']);
    Route::post('/register/submit', [UserController::class, 'registerSubmit']);
    Route::post('/login/new', [UserController::class, 'loginSubmit']);
    Route::post('/deleteaccount', [UserController::class, 'deleteaccount']);
    Route::post('/register', [UserController::class, 'register']);
    Route::post('/login', [UserController::class, 'login']);
    Route::post('/email-login', [UserController::class, 'loginWithEmail']);
    Route::get('/logout', [UserController::class, 'logout']);
    Route::get('/profile', [UserController::class, 'profile']);
    Route::post('/profile-update', [UserController::class, 'updateProfile']);
    Route::post('/profile-update-new', [UserController::class, 'updateProfileNew']);
    Route::post('/profile-update-photo', [UserController::class, 'updateProfilePhotoNew']);

    // News
    Route::post('/create-news-category', [NewsController::class, 'createNewsCategory']);
    Route::get('/list-news-category', [NewsController::class, 'listNewsCategory']);
    Route::post('/list-selected-news-category', [NewsController::class, 'listSelectedNewsCategory']);
    Route::get('/edit-news-category/{id}', [NewsController::class, 'editNewsCategory']);
    Route::get('/edit-news-count/{id}', [NewsController::class, 'editNewsCount']);
    Route::post('/update-news-category', [NewsController::class, 'updateNewsCategory']);
    Route::post('/update-news-count', [NewsController::class, 'updateNewsCount']);
    Route::get('/delete-news-category/{id}', [NewsController::class, 'deleteNewsCategory']);
    Route::post('/create-news', [NewsController::class, 'createNews']);
    Route::get('/list-news', [NewsController::class, 'listNews']);
    Route::get('/view-news/{id}', [NewsController::class, 'viewNews']);
    Route::get('/edit-news/{id}', [NewsController::class, 'editNews']);
    Route::post('/update-news', [NewsController::class, 'updateNews']);
    Route::get('/delete-news/{id}/{article}', [NewsController::class, 'deleteNews']);
    Route::post('/list-news-categorywise', [NewsController::class, 'listNewsCategoryWise']);
    Route::get('/bookmark-news/{id}', [NewsController::class, 'bookmarkNews']);
    Route::get('/remove-bookmark-news/{id}', [NewsController::class, 'removeBookmarkNews']);
    Route::get('/list-bookmark-news', [NewsController::class, 'listBookmarkNews']);
    Route::post('/update-user-category', [NewsController::class, 'updateUsercategory']);

    Route::post('/create-insight', [InsightController::class, 'createInsight']);
    Route::get('/view-insight/{id}', [InsightController::class, 'viewInsight']);
    Route::get('/list-insight', [InsightController::class, 'listInsight']);
    Route::get('/edit-insight/{id}', [InsightController::class, 'editInsight']);
    Route::post('/update-insight', [InsightController::class, 'updateInsight']);
    Route::get('/delete-insight/{id}', [InsightController::class, 'deleteInsight']);

    //poll routes
    Route::post('/create-poll', [PollController::class, 'createPoll']);
    Route::get('/list-poll', [PollController::class, 'listPoll']);
    Route::get('/edit-poll/{id}', [PollController::class, 'editPoll']);
    Route::post('/update-poll', [PollController::class, 'updatePoll']);
    Route::get('/delete-poll/{id}', [PollController::class, 'deletePoll']);
    Route::post('/list-poll-categorywise', [PollController::class, 'listPollCategoryWise']);
    Route::post('/poll-result', [PollController::class, 'PollResult']);
    Route::get('/view-poll/{id}', [PollController::class, 'viewPoll']);

    // Ads Routes
    Route::post('/create-ads', [AdsController::class, 'createAds']);
    Route::get('/list-ads', [AdsController::class, 'listAds']);
    Route::get('/delete-ads/{id}', [AdsController::class, 'deleteAds']);

    // Ads Routes
    Route::post('/create-banner', [BannerController::class, 'createBanner']);
    Route::get('/list-banner', [BannerController::class, 'listBanner']);
    Route::get('/delete-banner/{id}', [BannerController::class, 'deleteBanner']);

    // poll category routes
    Route::post('/create-course-category', [CourseController::class, 'createCourseCategory']);
    Route::get('/list-course-category', [CourseController::class, 'listCourseCategory']);
    Route::get('/edit-course-category/{id}', [CourseController::class, 'editCourseCategory']);
    Route::post('/update-course-category', [CourseController::class, 'updateCourseCategory']);
    Route::get('/delete-course-category/{id}', [CourseController::class, 'deleteCourseCategory']);

    Route::post('/create-course', [CourseController::class, 'createCourse']);
    Route::get('/list-course', [CourseController::class, 'listCourse']);
    Route::get('/list-course-categorywise/{id}', [CourseController::class, 'listCourseCategorywise']);
    Route::get('/view-course/{id}', [CourseController::class, 'viewCourse']);
    Route::get('/edit-course/{id}', [CourseController::class, 'editCourse']);
    Route::post('/update-course', [CourseController::class, 'updateCourse']);
    Route::get('/delete-course/{id}', [CourseController::class, 'deleteCourse']);

    Route::post('/create-lesson', [CourseController::class, 'createLesson']);
    Route::get('/list-lesson', [CourseController::class, 'listLesson']);
    Route::get('/list-lesson-categorywise/{id}', [CourseController::class, 'listLessonCategorywise']);
    Route::get('/edit-lesson/{id}', [CourseController::class, 'editLesson']);
    Route::post('/update-lesson', [CourseController::class, 'updateLesson']);
    Route::get('/delete-lesson/{id}', [CourseController::class, 'deleteLesson']);

    Route::post('/create-lectures', [CourseController::class, 'createLectures']);
    Route::get('/list-lectures', [CourseController::class, 'listLectures']);
    Route::get('/list-lectures-categorywise/{id}', [CourseController::class, 'listLecturesCategorywise']);
    Route::get('/edit-lectures/{id}', [CourseController::class, 'editLectures']);
    Route::post('/update-lectures', [CourseController::class, 'updateLectures']);
    Route::get('/delete-lectures/{id}', [CourseController::class, 'deleteLectures']);

    Route::get('/course-home-list', [CourseController::class, 'courseHomeList']);
    Route::get('/purchase-course/{id}', [CourseController::class, 'purchaseCourse']);
    Route::get('/my-courses', [CourseController::class, 'myPurchaseCourses']);
    Route::post('/free-courses', [CourseController::class, 'freeCourse']);
    Route::post('/paid-courses', [CourseController::class, 'paidCourse']);
    Route::get('/index', [HomeController::class, 'index']);


    Route::get('/marketApi', [MarketApiController::class, 'marketApi']);

    // holding user api
    Route::post('/create-holding-user-type', [HoldingsUserTypeController::class, 'createHoldingUserType']);
    Route::get('/list-holding-user-type', [HoldingsUserTypeController::class, 'listHoldingUserType']);
    Route::get('/edit-holding-user-type/{id}', [HoldingsUserTypeController::class, 'editHoldingUserType']);
    Route::post('/update-holding-user-type', [HoldingsUserTypeController::class, 'updateHoldingUserType']);
    Route::get('/delete-holding-user-type/{id}', [HoldingsUserTypeController::class, 'deleteHoldingUserType']);


    // holding user api
    Route::post('/create-holding-user', [HoldingsController::class, 'createHoldingUser']);
    Route::get('/list-holding-user', [HoldingsController::class, 'listHoldingUser']);
    Route::post('/list-holding-user-bytype', [HoldingsController::class, 'listHoldingUserBytype']);
    Route::get('/edit-holding-user/{id}', [HoldingsController::class, 'editHoldingUser']);
    Route::post('/update-holding-user', [HoldingsController::class, 'updateHoldingUser']);
    Route::get('/delete-holding-user/{id}', [HoldingsController::class, 'deleteHoldingUser']);

    // holding api
    Route::post('/create-holdings', [HoldingsController::class, 'createHoldings']);
    Route::get('/list-holdings', [HoldingsController::class, 'listHoldings']);
    Route::get('/view-holdings/{id}', [HoldingsController::class, 'viewHoldings']);
    Route::get('/edit-holdings/{id}', [HoldingsController::class, 'editHoldings']);
    Route::post('/update-holdings', [HoldingsController::class, 'updateHoldings']);
    Route::get('/delete-holdings/{id}', [HoldingsController::class, 'deleteHoldings']);

    // notification click data
    Route::post('notification-news-data',[NewsController::class,"notification_news_data"]);


    //WEALTH MANAGEMENT
    /* Currency */
    Route::get('currency-list',[WealthCurrencyController::class,'list']);
    Route::get('delete-currency/{id}',[WealthCurrencyController::class,'delete']);
    Route::post('create-currency',[WealthCurrencyController::class,'add']);
    Route::post('set-user-currency',[WealthUserCurrencyController::class,'add']);
    Route::post('get-user-currency',[WealthUserCurrencyController::class,'view']);

    /* USER ACCOUNT */
    Route::post('add-account',[WealthAccountController::class,'add']);
    Route::post('single-account-details',[WealthAccountController::class,'single']);
    Route::post('update-account',[WealthAccountController::class,'update']);
    Route::post('list-all-account',[WealthAccountController::class,'list']);
    /* Route::post('balance-transfer',[WealthAccountController::class,'transfer']);
    Route::post('update-balance-transfer',[WealthAccountController::class,'updateTransfer']);
    Route::post('delete-balance-transfer',[WealthAccountController::class,'deleteTransfer']);
    Route::post('account-history',[WealthAccountController::class,'history']); */

    Route::post('create-wallet',[WealthAccountController::class,'addWallet']);
    Route::post('list-wallet',[WealthAccountController::class,'listWallet']);
    Route::post('update-wallet',[WealthAccountController::class,'updateWallet']);
    Route::get('delete-wallet/{id}',[WealthAccountController::class,'delete']);

    /* CATEGORY */
    Route::post('add-category',[WealthCategoryController::class,'add']);
    Route::post('single-category-details',[WealthCategoryController::class,'single']);
    Route::post('update-category',[WealthCategoryController::class,'update']);
    Route::post('list-category',[WealthCategoryController::class,'list']);
    Route::get('delete-category/{id}',[WealthCategoryController::class,'delete']);
    Route::post('update-category-admin',[WealthCategoryController::class,'updateCategory']);
    Route::post('add-category-admin',[WealthCategoryController::class,'addCategory']);

    /* CATEGORY COLOR */
    Route::post('create-color',[WealthColorController::class,'add']);
    Route::post('single-color-details',[WealthColorController::class,'single']);
    Route::post('update-color',[WealthColorController::class,'update']);
    Route::post('color-list',[WealthColorController::class,'list']);
    Route::get('delete-color/{id}',[WealthColorController::class,'delete']);

    /* CATEGORY ICON */
    Route::post('create-icon',[WealthIconController::class,'add']);
    Route::post('single-icon-details',[WealthIconController::class,'single']);
    Route::post('update-icon',[WealthIconController::class,'update']);
    Route::post('list-icon',[WealthIconController::class,'list']);
    Route::get('delete-icon/{id}',[WealthIconController::class,'delete']);

    /* Transaction */
    Route::post('list-transaction',[WealthTransactionController::class,'list']);
    Route::post('add-transaction',[WealthTransactionController::class,'add']);
    Route::post('list-transaction-bydate',[WealthTransactionController::class,'listByDate']);
    Route::post('list-transaction-bycategory',[WealthTransactionController::class,'listByCategory']);
    Route::post('list-transaction-bydate-account',[WealthTransactionController::class,'listByDateAccount']);
    Route::post('list-transaction-byaccount',[WealthTransactionController::class,'listByAccount']);
    Route::post('single-transaction',[WealthTransactionController::class,'single']);
    Route::post('update-transaction',[WealthTransactionController::class,'update']);
    Route::post('delete-transaction',[WealthTransactionController::class,'delete']);

    /* SPLIT CATEGORY */
    Route::post('add-split-category',[WealthSplitCategoryController::class,'add']);
    Route::post('single-split-category-details',[WealthSplitCategoryController::class,'single']);
    /* Route::post('update-split-category',[WealthSplitCategoryController::class,'update']); */
    Route::post('list-split-category',[WealthSplitCategoryController::class,'list']);
    Route::get('delete-split-category/{id}',[WealthSplitCategoryController::class,'delete']);
    Route::post('update-split-category-admin',[WealthSplitCategoryController::class,'updateCategory']);
    Route::post('add-split-category-admin',[WealthSplitCategoryController::class,'addCategory']);

    /* SPLIT EVENT */
    Route::post('add-event',[WealthSplitEventController::class,'add']);
    Route::post('list-event',[WealthSplitEventController::class,'list']);
    Route::post('single-event-details',[WealthSplitEventController::class,'single']);
    Route::post('update-event',[WealthSplitEventController::class,'update']);
    Route::post('delete-event',[WealthSplitEventController::class,'delete']);
    Route::get('share-event/{event_id}/{user_id}',[WealthSplitEventController::class,'shareevent']);
    Route::post('share-event-member',[WealthSplitEventController::class,'shareEventMember']);

    /* SPLIT EVENT MEMBER */
    Route::post('add-event-member',[WealthSplitEventMemberController::class,'add']);
    Route::post('delete-event-member',[WealthSplitEventMemberController::class,'delete']);
    Route::post('list-event-member',[WealthSplitEventMemberController::class,'list']);

    /* SPLIT TRANSACTION */
    Route::post('add-split-transaction',[WealthSplitTransactionController::class,'add']);
    Route::post('update-split-transaction',[WealthSplitTransactionController::class,'update']);
    Route::post('delete-split-transaction',[WealthSplitTransactionController::class,'delete']);
    Route::post('list-split-transaction',[WealthSplitTransactionController::class,'list']);
    Route::post('single-split-transaction',[WealthSplitTransactionController::class,'singleTransaction']);
    Route::post('view-split-transaction',[WealthSplitTransactionController::class,'view']);
});

// Route::middleware('auth:sanctum')->get('/user', function (Request $request) {
//     return $request->user();
// });
